// ===========================================================================
// CONTENT  : CLASS AdminClient
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 08/08/2006
// HISTORY  :
//  08/08/2006  mdu  CREATED
//
// Copyright (c) 2006, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.ConnectException;
import java.net.Socket;
import java.net.UnknownHostException;

import org.pf.text.CommandLineArguments;
import org.pf.text.StringUtil;

/**
 * This admin client is capable of sending command line commands to the
 * Beluga administration server
 *
 * @author M.Duchrow
 * @version 1.0
 */
public class AdminClient
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final String PROG_VERSION		= "1.0.0" ;
	protected static final String PROG_NAME				= "Beluga Admin Client" ;
	protected static final String PROG_SIGNATURE	= PROG_NAME + " V" + PROG_VERSION ;
	
	protected static final int DEFAULT_BELUGA_ADMIN_PORT = 3510 ;
	protected static final String END_OF_COMMAND		= ";" ;

	protected static final String OPTION_COMMAN	= "-c" ;
	
	private static final StringUtil SU = StringUtil.current();
	
  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private int adminPort = DEFAULT_BELUGA_ADMIN_PORT ;
  protected int getAdminPort() { return adminPort ; }
  protected void setAdminPort( int newValue ) { adminPort = newValue ; }
  
  private String hostname = "localhost" ;
  protected String getHostname() { return hostname ; }
  protected void setHostname( String newValue ) { hostname = newValue ; }
  
  // =========================================================================
  // CLASS METHODS
  // =========================================================================
	public static void main(String[] args)
	{
		AdminClient client;
		CommandLineArguments options;

		client = new AdminClient();
		options = new CommandLineArguments(args);
		client.run(options);
		System.out.flush();
		System.exit(0);
	} // main() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	/**
	 * Initialize the new instance with default values.
	 */
	public AdminClient()
	{
		super();
	} // AdminClient() 

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	public void run(CommandLineArguments args)
	{
		String[] elements;
		String command;

		this.println(PROG_SIGNATURE);

		elements = args.getOptionValues(OPTION_COMMAN);
		if (SU.isNullOrEmpty(elements))
		{
			this.println("No command specified after option " + OPTION_COMMAN);
			this.showUsage();
			return;
		}
		command = SU.asString(elements, " ");
		if (SU.isNullOrBlank(command))
		{
			this.println("Commands must be indicated by option " + OPTION_COMMAN);
			this.showUsage();
			return;
		}

		this.println("Command: " + command);
		try
		{
			this.executeCommand(command);
		}
		catch (UnknownHostException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (ConnectException e)
		{
			this.println("Unable to connect to Beluga Admin Server. Maybe Beluga not started at all.");
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	} // run() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected void executeCommand(String command) throws UnknownHostException, IOException
	{
		Socket socket;
		OutputStream stream;
		Writer writer;

		socket = new Socket(this.getHostname(), this.getAdminPort());
		stream = socket.getOutputStream();
		writer = new PrintWriter(stream);
		writer.write(command);
		writer.write(END_OF_COMMAND);
		writer.flush();
		writer.close();
		stream.close();
		socket.close();
	} // executeCommand() 

	// -------------------------------------------------------------------------

	protected void showUsage()
	{
		this.println("Usage:");
		this.println("  AdminClient [-h <hostname>] [-p <port>] -c <command>");
		this.println("    -h <hostname>   Specifies the hostname where Beluga is running. Default is localhost.");
		this.println("    -p <port>       Specifies the port where Beluga is listening for admin commands. Default is 3510.");
		this.println("    -c <command>    Specifies the command to be sent to Beluga.");
		this.println("      supported commands are:");
		this.println("        shutdown          Terminates Beluga");
		this.println("        logLevel <level>  Changes the log level. Supported values are: ERROR, WARNING, INFO, DEBUG, NONE");
	} // showUsage()

	// -------------------------------------------------------------------------

	protected void print(String text)
	{
		System.out.print(text);
	} // print() 

	// -------------------------------------------------------------------------

	protected void println(String text)
	{
		System.out.println(text);
	} // println() 

	// -------------------------------------------------------------------------

} // class AdminClient 
