// ===========================================================================
// CONTENT  : CLASS Beluga
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.3.1 - 04/10/2011
// HISTORY  :
//  25/06/2004  mdu  CREATED
//	08/08/2006	mdu		added		-->	adminServer
//	16/05/2010	mdu		changed	--> to better support embedded usage
//	04/10/2011	mdu		changed	--> bugfix in Directory
//
// Copyright (c) 2004-2011, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.pfsw.tools.beluga.common.Constants.*;

import java.io.IOException;
import java.lang.management.ManagementFactory;
import java.util.Properties;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.pf.file.FileLocator;
import org.pf.file.PropertyFileLoader;
import org.pf.text.CommandLineArguments;
import org.pf.text.StringUtil;
import org.pf.util.SysUtil;
import org.pfsw.tools.beluga.admin.AdminMBean;
import org.pfsw.tools.beluga.admin.AdminServer;
import org.pfsw.tools.beluga.admin.AdminServerConfiguration;
import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.server.LDAPServer;
import org.pfsw.tools.beluga.ldap.server.LDAPServerConfiguration;

/**
 * The LDAP server main class.
 *
 * @author Manfred Duchrow
 * @version 1.3.1
 */
public class Beluga
{
	// =========================================================================
	// CONSTANTS
	// =========================================================================
	private static final StringUtil SU = StringUtil.current();
	
	public static final String PROG_VERSION = "1.0.0";
	protected static final String PROG_NAME = "Beluga";
	protected static final String PROG_SIGNATURE = PROG_NAME + " V" + PROG_VERSION;

	protected static final String SETTINGS_FILENAME = "beluga.properties";

	public static final String PARAM_SERVER_DEFINITION = "-s";
	public static final String PARAM_DIRECTORY_DEFINITION = "-d";
	public static final String PARAM_INTERCEPTORS = "-i";
	public static final String PARAM_LDAP_PORT = "-p";
	public static final String PARAM_ADMIN_PORT = "-ap";

	protected static final String ADMIN_MBEAN_NAME_PREFIX = "PFSW:type=Beluga-LDAP,port=";

	// =========================================================================
	// INSTANCE VARIABLES
	// =========================================================================
	private LDAPServer ldapServer;
	private AdminServer adminServer;
	private ObjectName adminMBeanName;

	// =========================================================================
	// PUBLIC CLASS METHODS
	// =========================================================================
	public static void main(String[] args)
	{
		Beluga inst;
		CommandLineArguments arguments;

		arguments = new CommandLineArguments(args);
		inst = new Beluga();
		inst.execute(arguments, false);
		SysUtil.current().exit(0);
	} // main() 

	// -------------------------------------------------------------------------

	/**
	 * Launches an LDAP server instance listening to the default port 389.
	 * and containing the data specified by the default definition file "directory.xml".
	 * <p>
	 * This method is supposed to be used by applications that want to use the
	 * LDAP server embedded.
	 */
	public static Beluga launch()
	{
		return launch(DEFAULT_DIRECTORY_DEFINITION, DEFAULT_LDAP_PORT);
	} // launch() 

	// -------------------------------------------------------------------------

	/**
	 * Launches an LDAP server instance listening to the given port.
	 * and containing the data specified by the default definition file "directory.xml".
	 * <p>
	 * This method is supposed to be used by applications that want to use the
	 * LDAP server embedded.
	 *
	 * @param dirPort The port of the LDAP server
	 */
	public static Beluga launch(int dirPort)
	{
		return launch(DEFAULT_DIRECTORY_DEFINITION, dirPort);
	} // launch() 

	// -------------------------------------------------------------------------

	/**
	 * Launches an LDAP server instance listening to the given default port 389
	 * and containing the data specified by the definition file given in by
	 * parameter dirDefinition.
	 * <p>
	 * This method is supposed to be used by applications that want to use the
	 * LDAP server embedded.
	 *
	 * @param dirDefinition The path of the XML file that contains the definition of
	 * files to be loaded into the LDAP server.
	 */
	public static Beluga launch(String dirDefinition)
	{
		return launch(dirDefinition, DEFAULT_LDAP_PORT);
	} // launch() 

	// -------------------------------------------------------------------------

	/**
	 * Launches an LDAP server instance listening to the given port and containing
	 * the data specified by the definition file given in by parameter dirDefinition.
	 * <p>
	 * This method is supposed to be used by applications that want to use the
	 * LDAP server embedded.
	 *
	 * @param dirDefinition The path of the XML file that contains the definition of
	 * files to be loaded into the LDAP server.
	 * @param dirPort The port of the LDAP server
	 * @throws IllegalArgumentException If the given port is out of allowed port range (1-65535)
	 */
	public static Beluga launch(String dirDefinition, int dirPort)
	{
		Beluga beluga;
		CommandLineArguments arguments;

		if ((dirPort <= 0) || (dirPort > 65535))
		{
			throw new IllegalArgumentException("dirPort must be greater than 0 and less than 65535!");
		}
		if ((!SysUtil.current().isWindows()) && (dirPort <= 1024)) 
		{
			throw new IllegalArgumentException("On Unix systems the dirPort must be greater than 1024, but it is " + dirPort);			
		}

		beluga = new Beluga();
		arguments = new CommandLineArguments();
		arguments.addOptionWithArgument(PARAM_DIRECTORY_DEFINITION, dirDefinition);
		arguments.addOptionWithArgument(PARAM_LDAP_PORT, Integer.toString(dirPort));
		beluga.execute(arguments, true);
		return beluga;
	} // launch() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	/**
	 * Initialize the new instance with default values.
	 */
	public Beluga()
	{
		super();
	} // Beluga() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Start the LDAP server and - if rundEmbedded is false - its associated admin server.
	 * The ports and files they are based on can be specified via the
	 * command line arguments.
	 * Parameter runsEmbedded must be set to true if the LDAP server is embedded
	 * in another application and therefore the current thread must not be
	 * blocked by it.
	 * If runsEmbedded is set to false the current thread is looping until
	 * an external signal shuts down the server.
	 */
	public void execute(CommandLineArguments args, boolean runsEmbedded)
	{
		this.setup(args, runsEmbedded);
		this.startLdapRequestProcessing();
		if (!runsEmbedded)
		{
			this.startAdminRequestProcessing();
		}
	} // execute() 

	// -------------------------------------------------------------------------

	/**
	 * Shutdown all servers
	 */
	public void shutdown()
	{
		if (adminServer != null)
		{
			adminServer.shutdownLdapServer();
			adminServer.shutdown();
		}
		else
		{
			this.ldapServer.stop();
		}
	} // shutdown() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected void setup(CommandLineArguments args, boolean runsEmbedded)
	{
		this.logger().logInfo(I_SERVER_STARTED, "{0} started.", PROG_SIGNATURE);
		this.initialize();
		this.setupLdapServer(args);
		if (!runsEmbedded)
		{
			this.setupAdminServer(args);
			this.registerAdminMBean();
		}
	} // setup() 

	// -------------------------------------------------------------------------

	protected void initialize()
	{
		Properties props;

		props = PropertyFileLoader.loadProperties(SETTINGS_FILENAME);
		if (props == null)
		{
			this.logger().logWarning(W_SETTINGS_NOT_FOUND, "Configuration file named <{0}> not found.", SETTINGS_FILENAME);
			return;
		}
		this.logger().initializePrintStreamLogging(props);
	} // initialize() 

	// -------------------------------------------------------------------------

	/**
	 * Must be called after setupLdapServer() because it needs a reference to the
	 * initialized LDAPServer
	 */
	protected void setupAdminServer(CommandLineArguments args)
	{
		AdminServerConfiguration config;
		String str;

		config = new AdminServerConfiguration();
		str = args.getArgumentValue(PARAM_ADMIN_PORT);
		if (!SU.isNullOrBlank(str))
		{
			config.setPort(Integer.parseInt(str));
		}
		adminServer = new AdminServer(config, ldapServer);
	} // setupAdminServer() 

	// -------------------------------------------------------------------------

	protected void registerAdminMBean()
	{
	  MBeanServer server;
	  AdminMBean mbean ;
	  
	  mbean = new AdminMBean(this.adminServer) ;
    server = ManagementFactory.getPlatformMBeanServer();
    try
    {
    	adminMBeanName = new ObjectName(ADMIN_MBEAN_NAME_PREFIX + ldapServer.getPort()) ;
      server.registerMBean(mbean, adminMBeanName);
    }
    catch ( Exception e )
    {
    	this.logger().logError(E_FAILED_ADMIN_MBEAN_REGISTRATION, "Failed to register administration MBean: {0}", e.toString());
    }		
	} // registerAdminMBean()

	// -------------------------------------------------------------------------
	
	protected void setupLdapServer(CommandLineArguments args)
	{
		LDAPServerConfiguration config;

		config = this.createConfiguration(args);
		try
		{
			ldapServer = new LDAPServer(config);
		}
		catch (IOException e)
		{
			this.logger().logError(E_FAILED_TO_START_LDAP_SERVER, "Failed to start server: {0}", e.toString());
		}
	} // setupLdapServer() 
	
	// -------------------------------------------------------------------------

	protected LDAPServerConfiguration createConfiguration(CommandLineArguments args)
	{
		LDAPServerConfiguration config;
		String str;

		config = new LDAPServerConfiguration();
		str = args.getArgumentValue(PARAM_SERVER_DEFINITION);
		if (!SU.isNullOrBlank(str))
		{
			config.initFrom(FileLocator.create(str));
		}
		// Now overwrite the config from the main config xml file if necessary
		str = args.getArgumentValue(PARAM_LDAP_PORT);
		if (!SU.isNullOrBlank(str))
		{
			config.setPort(Integer.parseInt(str));
		}
		str = args.getArgumentValue(PARAM_DIRECTORY_DEFINITION);
		if (!SU.isNullOrBlank(str))
		{
			config.setDirConfigUrl(FileLocator.create(str));
		}
		str = args.getArgumentValue(PARAM_INTERCEPTORS);
		if (!SU.isNullOrBlank(str))
		{
			config.setRespInterceptorsConfigUrl(FileLocator.create(str));
		}
		return config;
	} // createConfiguration() 

	// -------------------------------------------------------------------------

	/**
	 * Now start the listening loop on the configured admin port in the
	 * current thread.
	 */
	protected void startAdminRequestProcessing()
	{
		if (adminServer != null)
		{
			adminServer.start();
		}
	} // startAdminRequestProcessing() 

	// -------------------------------------------------------------------------

	/**
	 * Now start the listening loop on the configured LDAP port.
	 */
	protected void startLdapRequestProcessing()
	{
		Thread ldapServerThread;

		if (ldapServer != null)
		{
			ldapServerThread = new Thread(ldapServer, "LDAPServer(" + ldapServer.getPort() + ")");
			ldapServerThread.start();
		}
	} // startLdapRequestProcessing() 

	// -------------------------------------------------------------------------

	protected LoggerService logger()
	{
		return LoggerService.getLogger();
	} // logger() 

	// -------------------------------------------------------------------------

} // class Beluga 
