// ===========================================================================
// CONTENT  : CLASS Directory
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1.1 - 04/10/2011
// HISTORY  :
//  11/07/2004  mdu  CREATED
//	18/06/2006	mdu		changed	-->	Now based on a DirectoryConfiguration
//	04/10/2011	mdu		bugfix	--> in doAddModification()
//
// Copyright (c) 2004-2011, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.directory ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import org.pf.directory.DistinguishedName;
import org.pf.directory.DistinguishedNameElement;
import org.pf.directory.MultiValueAttribute;
import org.pf.directory.ldap.LDAPDirEntry;
import org.pf.directory.ldap.LDAPDirEntryContainer;
import org.pf.directory.ldap.LDAPDirEntryFilter;
import org.pf.directory.ldap.LDAPSearchResult;
import org.pf.directory.ldap.RDNFilter;
import org.pf.directory.metadata.AttributeMetaData;
import org.pf.directory.metadata.AttributesMetaData;
import org.pf.osf.MapFacade;
import org.pf.text.LdapFilterParser;
import org.pf.text.MatchRule;
import org.pf.text.MatchRuleParseException;
import org.pf.text.MatchRulePrinter;
import org.pf.text.StringUtil;
import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.messages.AddRequest;
import org.pfsw.tools.beluga.ldap.messages.AddResponse;
import org.pfsw.tools.beluga.ldap.messages.AttributeModificationDefinition;
import org.pfsw.tools.beluga.ldap.messages.BindRequest;
import org.pfsw.tools.beluga.ldap.messages.BindResponse;
import org.pfsw.tools.beluga.ldap.messages.DeleteRequest;
import org.pfsw.tools.beluga.ldap.messages.DeleteResponse;
import org.pfsw.tools.beluga.ldap.messages.ModifyRequest;
import org.pfsw.tools.beluga.ldap.messages.ModifyResponse;
import org.pfsw.tools.beluga.ldap.messages.ResponseMessage;
import org.pfsw.tools.beluga.ldap.messages.SearchRequest;
import org.pfsw.tools.beluga.ldap.messages.SearchResponse;
import org.pfsw.tools.beluga.ldap.messages.SearchResultDone;
import org.pfsw.tools.beluga.ldap.messages.SearchResultEntry;
import org.pfsw.tools.beluga.ldap.server.ContainerNotFoundException;
import org.pfsw.tools.beluga.ldap.server.InvalidDNException;
import org.pfsw.tools.beluga.ldap.server.IsNotContainerException;
import org.pfsw.tools.beluga.ldap.server.LDAPProcessingException;
import org.pfsw.tools.beluga.ldap.server.LDAPv3RequestProcessor;
import org.pfsw.tools.beluga.ldap.server.UndefinedSuffixException;

/**
 * Contains all data of a directory in memory. Allows to execute LDAP requests
 * against that data.
 *
 * @author Manfred Duchrow
 * @version 1.1.1
 */
public class Directory extends LDAPv3RequestProcessor
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final String E_OPEN_LDIF							= "DIR0001" ;
	protected static final String E_LOADING_LDIF					= "DIR0002" ;
	protected static final String E_IS_NO_CONTAINER				= "DIR0003" ;
	protected static final String E_CONTAINER_NOT_FOUND		= "DIR0004" ;
	protected static final String E_UPDATE_EXCEPTION			= "DIR0005" ;
	protected static final String E_FILTER_PARSING				= "DIR0006" ;
	protected static final String E_INVALID_SCOPE					= "DIR0007" ;
	protected static final String E_SEARCH_EXCEPTION			= "DIR0008" ;
	protected static final String E_RULE_PARSING					= "DIR0009" ;
	protected static final String E_INVALID_URL						= "DIR0010" ;
	protected static final String I_LOADED_LDIF_URL				= "DIR0011" ;
	protected static final String E_NO_SUFFIX_FOR_DN			= "DIR0012" ;
	protected static final String E_UNDEFINED_SUFFIX			= "DIR0013" ;
	protected static final String E_DELETE_ERROR					= "DIR0014" ;
	protected static final String E_CONVERT_TO_CONTAINER	= "DIR0015" ;
	protected static final String E_LOOKUP_ENTRY					= "DIR0016" ;
	protected static final String E_BIND_EXCEPTION				= "DIR0017" ;
	protected static final String E_MODIFY_EXCEPTION			= "DIR0018" ;

	protected static final LDAPDirEntry[] EMPTY_ENTRY_ARRAY = new LDAPDirEntry[0];

	protected static final String[] SINGLE_VALUE_ATTRNAMES =
	{
		LDAPDirEntry.ATTRNAME_CREATE_TIMESTAMP,
		LDAPDirEntry.ATTRNAME_MODIFY_TIMESTAMP,
		LDAPDirEntry.ATTRNAME_CREATORS_NAME,
		LDAPDirEntry.ATTRNAME_MODIFIERS_NAME
	} ;

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private LDAPDirEntryContainer rootContainer ;
  protected LDAPDirEntryContainer getRootContainer() { return rootContainer ; }
  protected void setRootContainer( LDAPDirEntryContainer newValue ) { rootContainer = newValue ; }

  private MatchRule containerRule = null ;
  protected MatchRule getContainerRule() { return containerRule ; }
  protected void setContainerRule( MatchRule newValue ) { containerRule = newValue ; }

  private AttributesMetaData metaData = null ;
  protected AttributesMetaData getMetaData() { return metaData ; }
  protected void setMetaData( AttributesMetaData newValue ) { metaData = newValue ; }
  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with a configuration.
   */
  public Directory( DirectoryConfiguration config )
  {
    super() ;
    this.initRoot() ;
    this.initMetaData() ;
    this.init( config ) ;
  } // Directory()

  // -------------------------------------------------------------------------

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
	/**
	 * Executes a LDAPv3 bindRequest.
	 * Returns a bindResponse or null if it doesn't handle the request.
	 *
	 * @param request The bind request message.
	 */
  public BindResponse bind( BindRequest request )
  {
  	BindResponse response ;
  	LDAPDirEntry entry ;
  	String userPwd ;
  	HashedPassword userPassword ;

		response = new BindResponse( request ) ;
		if ( this.str().isNullOrBlank( request.getDN() ) )
		{
			response.inappropriateAuthentication() ;
			return response ;
		}
		try
		{
			entry = this.findByDN( request.getDN() ) ;
			if ( entry == null )
			{
				response.objectNotFound( request.getDN() ) ;
			}
			else
			{
				userPwd = (String)entry.getValue( this.getPasswordAttributeName() ) ;
				this.logger().logDebug( "User pwd: {0}", userPwd ) ;
				// TODO Remove debug log for clear text password
				this.logger().logDebug( "Bind pwd: {0}", new String(request.getPassword()) ) ;
				userPassword = new HashedPassword( userPwd ) ;
				if ( userPassword.isEqual( request.getPassword() ) )
				{
					// Nothing to do - response has success as default
				}
				else
				{
					response.invalidPassword( request.getDN() ) ;
				}
			}
		}
		catch ( LDAPProcessingException e )
		{
			this.exceptionToResponse( e, response ) ;
		}
		catch ( RuntimeException e )
		{
			this.logger().logError( E_BIND_EXCEPTION, "Internal error in BIND", e ) ;
			response.operationsError( e.getMessage() ) ;
		}
		return response ;
  } // bind()

  // -------------------------------------------------------------------------

	/**
	 * Executes a LDAPv3 searchRequest.
	 * Returns an array of search response objects or null if it doesn't
	 * handle the request.
	 *
	 * @param request The search request message.
	 */
  public SearchResponse[] search( SearchRequest request )
  {
  	LDAPDirEntry baseObject ;
  	LDAPSearchResult result ;
  	LDAPDirEntryContainer container ;
  	SearchResponse[] responses = null ;
  	String baseDN ;

  	baseDN = request.getBaseDN() ;
  	if ( baseDN == null )
		{
  		responses = this.createSearchOperationsError( request ,
  				"No base DN specified for search request: " + request ) ;
  		return responses ;
		}

  	if ( baseDN.length() == 0 )
  	{
  		responses = this.createSearchResponse( request, this.getRootContainer() ) ;
  		return responses ;
  	}

  	try
		{
			baseObject = this.findByDN( baseDN ) ;
			if ( baseObject == null )
			{
				return this.baseDNnotFoundResponse( request ) ;
			}

			if ( request.isScopeBaseObject() )
			{
				responses = this.createSearchResponse( request, baseObject ) ;
			}
			else
			{
				if ( ( request.isScopeSingleLevel() ) || ( request.isScopeWholeSubtree() ) )
				{
					if ( baseObject.isContainer() )
					{
						container = (LDAPDirEntryContainer)baseObject ;
						try
						{
							if ( request.isScopeSingleLevel() )
							{
								result = container.findChildren( request.getFilter(),
																									request.getSizeLimit() ) ;
							}
							else
							{
								if ( this.logger().isDebugOn() )
								{
									MatchRule rule ;
							  	rule = LdapFilterParser.parseFilter( request.getFilter() ) ;
							  	rule.ignoreCase( true ) ;
							  	rule.ignoreCaseInNames( true ) ;
							  	MatchRulePrinter mrPrinter = new MatchRulePrinter() ;
							  	mrPrinter.useNewSyntax( true ) ;
							  	this.logger().logDebug( "Search: " + mrPrinter.asString(rule) ) ;
								}
								result = container.findAll( request.getFilter(),
																										request.getSizeLimit() ) ;
							}
							responses = this.createSearchResponse( request, result ) ;
						}
						catch ( MatchRuleParseException e1 )
						{
							this.logger().logError( E_FILTER_PARSING, "Error parsing filter {0}",
									request.getFilter(), e1) ;
							responses = this.createSearchDoneResponse( request ) ;
							responses[0].operationsError( e1.getMessage() ) ;
						}
					}
					else
					{
						responses = this.createEmptyResponse( request ) ;
					}
				}
				else
				{
					this.logger().logError( E_INVALID_SCOPE, "Invalid search scope: {0}",
																	Integer.valueOf( request.getScope() ) ) ;
					responses = this.createSearchOperationsError( request ,
														"Invalid search scope: " + request.getScope() ) ;
				}
			}
		}
		catch ( LDAPProcessingException e )
		{
			this.logger().logError( E_SEARCH_EXCEPTION,
					"Search failed to execute: {0}", request ) ;
			responses = this.createSearchOperationsError( request ,
				"Exception occurred when executing a search request: " + request ) ;
		}
		catch ( RuntimeException e )
		{
			this.logger().logError( E_SEARCH_EXCEPTION,	"Runtime exception in SEARCH", e ) ;
			responses = this.createSearchOperationsError( request ,
					"Exception occurred when executing a search request: " + request ) ;
		}

  	return responses ;
  } // search()

  // -------------------------------------------------------------------------

  /**
   * Executes a LDAPv3 DeleteRequest.
   *
   * @param request The delete request message.
   */
  public DeleteResponse delete( DeleteRequest request )
  {
  	DeleteResponse response ;
  	LDAPDirEntry entry ;
  	LDAPDirEntryContainer container ;

  	response = new DeleteResponse( request ) ;
  	entry = this.locateByDN( request.getDN() ) ;
  	if ( entry == null  )
		{
			response.noSuchObject( "", "Unable to find object with dn: " + request.getDN() ) ;
			return response ;
		}
  	if ( ! request.getDN().equals( entry.getDistinguishedName() ) )
  	{
  		response.noSuchObject( entry.getDN(), "Unable to find object with dn: " + request.getDN() ) ;
  		return response ;
  	}
  	if ( entry.isContainer() )
		{
  		container = (LDAPDirEntryContainer)entry ;
  		if ( container.getChildCount() > 0 )
			{
  			response.notAllowedOnNonLeaf( container.getDN(), "Failed to delete entry " + request.getDN() ) ;
  			return response ;
			}
  		if ( container instanceof LDAPSuffix )
  		{
  			response.notAllowedOnNonLeaf( container.getDN(), "Suffix must not be deleted" ) ;
  			return response ;
  		}
		}
  	response.setDN( entry.getDN() ) ;
  	if ( ! this.basicDelete( entry ) )
		{
			response.unwillingToPerform( "Failed to delete entry " + request.getDN() ) ;
		}
  	return response ;
  } // delete()

  // -------------------------------------------------------------------------

  public AddResponse add( AddRequest request )
  {
  	AddResponse response ;
  	LDAPDirEntry entryToAdd ;
  	LDAPDirEntry parentEntry ;
  	LDAPDirEntryContainer parentContainer ;
  	DistinguishedName parentDN ;

  	response = new AddResponse( request ) ;
  	entryToAdd = request.getEntry() ;
  	entryToAdd.setMetaData( this.getMetaData() ) ;
  	parentDN = entryToAdd.getDistinguishedName().getParent() ;
  	parentEntry = this.locateByDN( parentDN ) ;
  	if ( ! parentDN.equals( parentEntry.getDistinguishedName() ) )
		{
			response.noSuchObject( parentDN.toString() ) ;
			return response ;
		}
  	if ( parentEntry.isContainer() )
  	{
  		parentContainer = (LDAPDirEntryContainer)parentEntry ;
  	}
  	else
		{
  		try
			{
				parentContainer = this.becomeContainer( parentEntry ) ;
			}
			catch ( LDAPProcessingException e )
			{
				this.logger().logError( E_CONVERT_TO_CONTAINER,
						"Error changing leaf object to container {0}", parentEntry.getDN(), e) ;
				response.unwillingToPerform( "Error changing leaf object to container: " + parentEntry.getDN() ) ;
				return response ;
			}
		}
  	if ( parentContainer.contains( entryToAdd.getDistinguishedName() ) )
		{
  		response.entryAlreadyExists( entryToAdd.getDistinguishedName(), "Entry cannot be added because it already exists" ) ;
		}
		else
		{
			if ( this.isContainer( entryToAdd ) )
			{
				entryToAdd = new LDAPDirEntryContainer( entryToAdd ) ;
			}
			parentContainer.addChild( entryToAdd ) ;
		}
  	return response ;
  } // add()

  // -------------------------------------------------------------------------

  public ModifyResponse modify( ModifyRequest request )
	{
  	ModifyResponse response ;
  	LDAPDirEntry entry ;

  	response = new ModifyResponse( request ) ;
  	try
		{
			entry = this.findByDN( request.getDN() ) ;
			if ( entry == null )
			{
				response.noSuchObject( request.getDN() ) ;
				return response ;
			}
		}
		catch ( LDAPProcessingException e )
		{
			this.logger().logError( E_LOOKUP_ENTRY,
					"Error when looking up entry to modify with dn: {0}", request.getDN(), e) ;
			response.unwillingToPerform( "Error when looking up entry to modify with dn: " + request.getDN() ) ;
			return response ;
		}
		catch ( RuntimeException e )
		{
			this.logger().logError( E_MODIFY_EXCEPTION, "Internal error in MODIFY", e ) ;
			response.operationsError( e.getMessage() ) ;
			return response ;
		}
		this.executeModificationsOnEntry( request, response, entry ) ;
  	return response ;
	} // modify()

	// -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected LDAPDirEntry findByDN( final String dn ) throws LDAPProcessingException
	{
		DistinguishedName distinguishedName ;

		distinguishedName = new DistinguishedName(dn) ;
		return this.findByDN( distinguishedName ) ;
	} // findByDN()

  // -------------------------------------------------------------------------

  protected LDAPDirEntry findByDN( final DistinguishedName distinguishedName )
  	throws LDAPProcessingException
	{
		LDAPDirEntry entry ;
		LDAPDirEntryContainer container ;
		DistinguishedNameElement[] dnParts ;

		entry = this.findSuffixWithDN( distinguishedName ) ;
		if ( entry != null )
		{
			return entry ;
		}
		container = this.findSuffixOf( distinguishedName ) ;
		if ( container == null )
		{
			this.logger().logError( E_NO_SUFFIX_FOR_DN, "No suffix found for object with dn: {0}",
					distinguishedName.toString() ) ;
			return null ;
		}

		dnParts = distinguishedName.getPartsWithout( container.getDistinguishedName().partCount() ) ;
		for (int i = dnParts.length-1; i >= 0 ; i-- )
		{
			entry = container.findChild( new RDNFilter( dnParts[i] ) ) ;
			if ( entry == null )
			{
				return null ;
			}
			if ( i == 0 )
			{
				return entry ;
			}
			if ( entry instanceof LDAPDirEntryContainer )
			{
				container = (LDAPDirEntryContainer)entry ;
			}
			else
			{
				throw new IsNotContainerException( entry.getDN() ) ;
			}
		}
		return entry ;
	} // findByDN()

	// -------------------------------------------------------------------------

  /**
   * Returns the suffix object the given DN belongs to or null
   */
  protected LDAPSuffix findSuffixOf( final DistinguishedName dn )
	{
		LDAPDirEntryFilter filter ;

		filter = new LDAPDirEntryFilter()
		{
			public boolean matches( LDAPDirEntry entry )
			{
				return entry.getDistinguishedName().isParentOf( dn );
			}
		};
		return (LDAPSuffix)this.getRootContainer().findChild( filter ) ;
	} // findSuffixOf()

	// -------------------------------------------------------------------------

  /**
   * Returns the matching suffix object or null
   */
  protected LDAPSuffix findSuffixWithDN( final DistinguishedName dn )
  {
  	LDAPDirEntryFilter filter ;

  	filter = new LDAPDirEntryFilter()
  	{
  		public boolean matches( LDAPDirEntry entry )
  		{
  			return entry.getDistinguishedName().equals( dn );
  		}
  	};
  	return (LDAPSuffix)this.getRootContainer().findChild( filter ) ;
  } // findSuffixWithDN()

  // -------------------------------------------------------------------------

  /**
   * Returns the last existing entry element that can be found in the hierarchy
   * by the given distinguishedName. In the best case it is exactly the entry
   * which has the same dn as the given one. In worst case null will be returned
   * if even vo matching suffix can be found.
   * This method is not writing anything to the log!
   */
  protected LDAPDirEntry locateByDN( final DistinguishedName distinguishedName )
	{
		LDAPDirEntry entry;
		LDAPDirEntryContainer container;
		DistinguishedNameElement[] dnParts;

		container = this.getSuffixFor( distinguishedName );
		if ( container == null )
		{
			return null;
		}
		entry = container ;
		dnParts = distinguishedName.getPartsWithout( container.getDistinguishedName().partCount() );
		for (int i = dnParts.length - 1; i >= 0; i-- )
		{
			entry = container.findChild( new RDNFilter( dnParts[i] ) );
			if ( entry == null )
			{
				return container;
			}
			if ( i == 0 )
			{
				return entry;
			}
			if ( entry instanceof LDAPDirEntryContainer )
			{
				container = (LDAPDirEntryContainer) entry;
			}
			else
			{
				return entry ;
			}
		}
		return entry;
	} // locateByDN()

	// -------------------------------------------------------------------------

  /**
   * Returns the suffix that is either directly identified by the given dn
   * or is the base of the element with the given dn.
   * Returns null if no such suffix can be found.
   */
  protected LDAPSuffix getSuffixFor( DistinguishedName dn )
	{
		LDAPSuffix suffix;

		suffix = this.findSuffixWithDN( dn );
		if ( suffix == null )
		{
			suffix = this.findSuffixOf( dn );
		}
		return suffix ;
	} // getSuffixFor()

	// -------------------------------------------------------------------------

  protected SearchResponse[] createEmptyResponse( SearchRequest request )
	{
		return this.createSearchDoneResponse( request ) ;
	} // createEmptyResponse()

	// -------------------------------------------------------------------------

  protected SearchResponse[] createSearchResponse( SearchRequest request, LDAPDirEntry entry )
	{
  	LDAPSearchResult result ;

  	result = new LDAPSearchResult(1) ;
  	result.add( entry ) ;
  	return this.createSearchResponse( request, result ) ;
	} // createSearchResponse()

  // -------------------------------------------------------------------------

  protected SearchResponse[] createSearchResponse( SearchRequest request, LDAPSearchResult result )
	{
  	SearchResponse[] responses = null ;
  	SearchResultEntry resultEntry ;
  	LDAPDirEntry[] objects ;
  	SearchResultDone doneResponse ;

  	objects = result.getFoundObjects() ;
		responses = new SearchResponse[objects.length+1] ;
		for (int i = 0; i < objects.length; i++ )
		{
			resultEntry = new SearchResultEntry( request ) ;
			resultEntry.setDN( objects[i].getDN() ) ;
			resultEntry.setAttributes( objects[i].getCopyOfAttributes( request.getAttributeNames() ) ) ;
			responses[i] = resultEntry ;
		}
		doneResponse = new SearchResultDone( request ) ;
		if ( result.isSizeLimitExceeded() )
		{
			doneResponse.setSizeLimitExceeded() ;
		}
		responses[objects.length] = doneResponse ;
		return responses ;
	} // createSearchResponse()

	// -------------------------------------------------------------------------

  protected SearchResponse[] createSearchDoneResponse( SearchRequest request )
	{
  	SearchResponse[] responses ;

		responses = new SearchResponse[1] ;
		responses[0] = new SearchResultDone( request ) ;
		return responses ;
	} // createSearchDoneResponse()

	// -------------------------------------------------------------------------

  protected SearchResponse[] createSearchOperationsError( SearchRequest request, String msg )
	{
  	SearchResponse[] responses ;

  	responses = this.createSearchDoneResponse( request ) ;
		responses[0].operationsError( msg ) ;
		return responses ;
	} // createSearchOperationsError()

	// -------------------------------------------------------------------------

  protected void loadFrom( DirectoryDataFile dataFile )
	{
  	LDIFDirectoryLoader loader ;

  	if ( dataFile.isLDIF() )
		{
  		loader = new LDIFDirectoryLoader( this.logger() ) ;
  		loader.loadInto( this, dataFile.getFileLocator() ) ;
		}
  	// TODO Support DSML loading
	} // loadFrom()

	// -------------------------------------------------------------------------

  /**
   * Returns the container object the given DN is child of or null if not found.
   *
   * @param dn The dn of a child object of the container to be looked up
   */
  protected LDAPDirEntryContainer findContainerFor( String dn )
  	throws LDAPProcessingException
	{
  	DistinguishedName distinguishedName ;

  	distinguishedName = new DistinguishedName( dn ) ;
  	return this.findContainerFor( distinguishedName ) ;
	} // findContainerFor()

  // -------------------------------------------------------------------------

  /**
   * Returns the container object the given DN is child of or null if not found.
   *
   * @param distinguishedName The dn of a child object of the container to be looked up
   */
  protected LDAPDirEntryContainer findContainerFor( final DistinguishedName distinguishedName )
  	throws LDAPProcessingException
	{
		LDAPDirEntry entry ;
		if ( distinguishedName.partCount() == 1 )
		{
			throw new UndefinedSuffixException( distinguishedName.toString() ) ;
		}
		entry = this.findByDN( distinguishedName.getParent() ) ;
		if ( entry == null )
		{
			throw new ContainerNotFoundException( distinguishedName.getParent().toString() ) ;
		}
		if ( entry instanceof LDAPDirEntryContainer )
		{
			return (LDAPDirEntryContainer)entry ;
		}
		else
		{
			throw new IsNotContainerException( entry.getDN() ) ;
		}
	} // findContainerFor()

	// -------------------------------------------------------------------------

  /**
   * Propagates the meaning/contents of the given exception to the specified
   * response message.
   */
  protected void exceptionToResponse( LDAPProcessingException exception,
  																		ResponseMessage response )
	{
		if ( exception instanceof InvalidDNException )
		{
			response.invalidDN( exception.getMessage() ) ;
		}
		else
		{
			response.operationsError( exception.getMessage() ) ;
		}
	} // exceptionToResponse()

	// -------------------------------------------------------------------------

  /**
   * Returns true if the given entry has any container objectclass
   */
  protected boolean isContainer( LDAPDirEntry entry )
	{
  	MapFacade mapFacade ;

  	mapFacade = new MapFacade( entry ) ;
		return this.getContainerRule().matches( mapFacade ) ;
	} // isContainer()

	// -------------------------------------------------------------------------

  protected SearchResponse[] baseDNnotFoundResponse( SearchRequest request )
	{
  	SearchResponse[] responses ;

  	responses = new SearchResponse[1] ;
  	responses[0] = new SearchResultDone( request ) ;
  	responses[0].noSuchObject( request.getBaseDN() ) ;
  	return responses ;
	} // baseDNnotFoundResponse()

	// -------------------------------------------------------------------------

  /**
   * Returns true if element has been successfully removed
   */
  protected boolean basicDelete( final LDAPDirEntry entry )
	{
		LDAPDirEntryContainer container ;

		try
		{
			container = this.findContainerFor( entry.getDistinguishedName() ) ;
			return container.removeChild( entry ) ;
		}
		catch ( Exception e )
		{
			this.logger().logError( E_DELETE_ERROR, "Exception {0} when trying to delete dn {1}",
					new Object[] { e.toString(), entry.getDistinguishedName() } ) ;
		}
		return false ;
	} // basicDelete()

	// -------------------------------------------------------------------------

  protected LDAPDirEntryContainer becomeContainer( LDAPDirEntry entry ) throws LDAPProcessingException
	{
		LDAPDirEntryContainer container ;
		LDAPDirEntryContainer parent ;

		container = new LDAPDirEntryContainer( entry ) ;
		parent = this.findContainerFor( entry.getDistinguishedName() ) ;
		parent.removeChild( entry ) ;
		parent.addChild( container ) ;
		return container ;
	} // becomeContainer()

	// -------------------------------------------------------------------------

	protected void executeModificationsOnEntry( ModifyRequest request, ModifyResponse response, LDAPDirEntry entry )
	{
		AttributeModificationDefinition[] modifications ;

		modifications = request.getModificationDefinitions();
		if ( modifications.length > 0 )
		{
			for (int i = 0; i < modifications.length; i++ )
			{
				if ( modifications[i].isAddOperation() )
				{
					this.doAddModification( entry, modifications[i] ) ;
				}
				else if ( modifications[i].isDeleteOperation() )
				{
					this.doDeleteModification( entry, modifications[i] ) ;
				}
				else if ( modifications[i].isReplaceOperation() )
				{
					this.doReplaceModification( entry, modifications[i] ) ;
				}
			}
			entry.updateModifyTimestamp() ;
		}
	} // executeModificationsOnEntry()

	// -------------------------------------------------------------------------

	protected void doAddModification( LDAPDirEntry entry, AttributeModificationDefinition modification )
	{
		MultiValueAttribute attribute ;
		MultiValueAttribute entryAttr ;

		attribute = modification.getAttribute();
		if ( attribute.valueCount() > 0 )
		{
			if ( entry.hasAttribute( attribute.getName() ) )
			{
				entryAttr = entry.getAttribute( attribute.getName() ) ;
			}
			else
			{
				entryAttr = new MultiValueAttribute( attribute.getName() ) ;
				entry.addAttribute(entryAttr);
			}
			entryAttr.addValues( attribute.getValues() ) ;
		}
	} // doAddModification()

	// -------------------------------------------------------------------------

	protected void doReplaceModification( LDAPDirEntry entry, AttributeModificationDefinition modification )
	{
		MultiValueAttribute attribute ;
		MultiValueAttribute entryAttr ;

		attribute = modification.getAttribute();
		if ( attribute.valueCount() > 0 )
		{
			this.doDeleteModification( entry, modification ) ;
			entryAttr = new MultiValueAttribute( attribute.getName() ) ;
			entryAttr.addValues( attribute.getValues() ) ;
			entry.addAttribute( entryAttr ) ;
		}
		else
		{
			this.doDeleteModification( entry, modification ) ;
		}
	} // doReplaceModification()

	// -------------------------------------------------------------------------

	protected void doDeleteModification( LDAPDirEntry entry, AttributeModificationDefinition modification )
	{
		MultiValueAttribute attribute ;

		attribute = modification.getAttribute() ;
		entry.removeAttribute( attribute.getName() ) ;
	} // doDeleteModification()

	// -------------------------------------------------------------------------

	protected void initRoot()
	{
		LDAPDirEntryContainer root ;

		root = new LDAPDirEntryContainer( "" ) ;
		root.setAttribute( LDAPDirEntry.ATTRNAME_NAMING_CONTEXTS, "" ) ;
		root.setAttribute( LDAPDirEntry.ATTRNAME_SUBSCHEMA_SUBENTRY, "" ) ;
		this.setRootContainer( root ) ;
	} // initRoot()

	// -------------------------------------------------------------------------

	protected void initMetaData()
	{
		AttributesMetaData md ;
		AttributeMetaData amd ;

		md = new AttributesMetaData() ;
		this.setMetaData( md ) ;

		for (int i = 0; i < SINGLE_VALUE_ATTRNAMES.length; i++ )
		{
			amd = new AttributeMetaData( SINGLE_VALUE_ATTRNAMES[i] ) ;
			amd.beSingleValued() ;
			md.setMetaData( amd ) ;
		}
	} // initMetaData()

	// -------------------------------------------------------------------------

  protected void init( DirectoryConfiguration config )
	{
  	this.setContainerRule( config.getContainerRule() ) ;
  	this.initSuffixes( config ) ;
  	this.initDataFromFiles( config );
		if ( this.logger().isDebugOn() )
		{
			//Inspector.inspect( this.getRootContainer() ) ;
		}
	} // init()

  // -------------------------------------------------------------------------

	protected void initDataFromFiles( DirectoryConfiguration config )
	{
		DirectoryDataFile[] dataFiles;
		dataFiles = config.getDataFiles() ;
  	for (int i = 0; i < dataFiles.length; i++ )
		{
			this.loadFrom( dataFiles[i] ) ;
		}
	} // initDataFromFiles()

	// -------------------------------------------------------------------------

  protected void initSuffixes( DirectoryConfiguration config )
	{
  	LDAPSuffix suffix ;
  	DistinguishedName[] suffixNames ;

  	suffixNames = config.getSuffixes() ;
  	for (int i = 0; i < suffixNames.length; i++ )
  	{
  		suffix = new LDAPSuffix( suffixNames[i] ) ;
  		suffix.setMetaData( this.getMetaData() ) ;
  		suffix.updateTimestamps() ;
  		this.getRootContainer().addChild( suffix ) ;
  	}
	} // initSuffixes()

	// -------------------------------------------------------------------------

  protected String getPasswordAttributeName()
	{
		return "userpassword" ;
	} // getPasswordAttributeName()

	// -------------------------------------------------------------------------

	protected LoggerService logger()
	{
		return LoggerService.getLogger() ;
	} // logger()

	// -------------------------------------------------------------------------

  protected StringUtil str()
	{
		return StringUtil.current();
	} // str()

	//-------------------------------------------------------------------------

} // class Directory
