// ===========================================================================
// CONTENT  : CLASS HashedPassword
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 14/07/2004
// HISTORY  :
//  14/07/2004  mdu  CREATED
//
// Copyright (c) 2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.directory;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

import org.pf.text.StringUtil;
import org.pf.util.Base64Converter;

// ===========================================================================
// IMPORTS
// ===========================================================================

/**
 * Container for a SHA hashed password including salt if it is SSHA.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class HashedPassword
{
	// =========================================================================
	// CONSTANTS
	// =========================================================================

	// =========================================================================
	// INSTANCE VARIABLES
	// =========================================================================
  private byte[] pwHash = null ;
  protected byte[] getPwHash() { return pwHash ; }
  protected void setPwHash( byte[] newValue ) { pwHash = newValue ; }
  
  private byte[] salt = null ;
  protected byte[] getSalt() { return salt ; }
  protected void setSalt( byte[] newValue ) { salt = newValue ; }  
  
  private boolean isSeeded = false ;
  protected boolean getIsSeeded() { return isSeeded ; }
  protected void setIsSeeded( boolean newValue ) { isSeeded = newValue ; }  

  private boolean isHashed = false ;
  protected boolean getIsHashed() { return isHashed ; }
  protected void setIsHashed( boolean newValue ) { isHashed = newValue ; }  
  
  // =========================================================================
	// CLASS METHODS
	// =========================================================================

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	/**
	 * Initialize the new instance with a hashed password.
	 * 
	 * @param password A base64 encoded hash of a password with a prefix {SHA} or {SSHA} 
	 */
	public HashedPassword( String password )
	{
		super();
		this.init( password ) ;
	} // HashedPassword() 

	// -------------------------------------------------------------------------
	
	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Compares the given clear text password with the hashed password.
	 * Returns true if the hash of the given password equals the password hash
	 * in this object.
	 */
	public boolean isEqual( byte[] password ) 
	{
		byte[] otherPassword ;
		byte[] otherHash ;
		MessageDigest sha ;
		
		if ( this.getIsHashed() )
		{
			if ( this.getIsSeeded() )
			{
				otherPassword = new byte[password.length + this.getSalt().length] ;
				System.arraycopy( password, 0, otherPassword, 0, password.length ) ;
				System.arraycopy( this.getSalt(), 0, otherPassword, password.length, this.getSalt().length ) ;
			}
			else
			{
				otherPassword = password ;
			}
			try
			{
				sha = MessageDigest.getInstance ("SHA-1");
			}
			catch ( NoSuchAlgorithmException e )
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				return false ;
			}
			otherHash = sha.digest( otherPassword ) ;
		}
		else
		{
			otherHash = password ;
		}
		return Arrays.equals( this.getPwHash(), otherHash ) ;
	} // isEqual()

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected void init( String password ) 
	{
		String prefix ;
		String base64Hash ;
		byte[] hash ;
		byte[][] hashAndSalt ;
		
		prefix = this.str().prefix( password, "}" ) ;
		if ( prefix == null ) // not hashed at all
		{
			this.setPwHash( password.getBytes() ) ;
			this.setIsHashed( false ) ;
			return ;
		}
		
		this.setIsHashed( true ) ;
		prefix = prefix.toUpperCase() + "}" ;
		base64Hash = this.str().suffix( password, "}" ) ;
		hash = Base64Converter.decode( base64Hash ) ;
		
		if ( prefix.equals( "{SHA}" ) )
		{
			this.setIsSeeded( false ) ;
			this.setPwHash( hash ) ;
		}
		else
		{
			if ( prefix.equals( "{SSHA}" ) )
			{
				this.setIsSeeded( true ) ;
				hashAndSalt = this.split( hash, 20 ) ; // 20 bytes = 160 bit
				this.setPwHash( hashAndSalt[0] ) ;
				this.setSalt( hashAndSalt[1] ) ;
			}
		}
	} // init()

	// -------------------------------------------------------------------------
	
	protected byte[][] split( byte[] src, int n )
	{
		byte[] l, r;
		if ( src.length <= n )
		{
			l = src;
			r = new byte[0];
		}
		else
		{
			l = new byte[n];
			r = new byte[src.length - n];
			System.arraycopy( src, 0, l, 0, n );
			System.arraycopy( src, n, r, 0, r.length );
		}
		byte[][] lr = { l, r };
		return lr;
	} // split()

	// -------------------------------------------------------------------------
	
	protected StringUtil str()
	{
		return StringUtil.current();
	} // str()

	//-------------------------------------------------------------------------
	
} // class HashedPassword 
