// ===========================================================================
// CONTENT  : CLASS XmlDirectoryConfiguration
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 18/06/2006
// HISTORY  :
//  18/06/2006  mdu  CREATED
//
// Copyright (c) 2006, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.directory ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import org.pf.file.FileLocator;
import org.pf.six.Element;

/**
 * Special configuration that gets initialized from XML based data.
 *
 * @author M.Duchrow
 * @version 1.0
 */
public class XmlDirectoryConfiguration extends DirectoryConfiguration
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final String TAG_ROOT								= "directory" ;
	protected static final String TAG_CONFIGURATION				= "configuration" ;
	protected static final String TAG_CONTAINER_RULE			= "container-rule" ;
	protected static final String TAG_META_DATA						= "meta-data" ;
	protected static final String TAG_DATA								= "data" ;
	protected static final String TAG_SUFFIX							= "suffix" ;
	protected static final String TAG_DATA_SOURCE					= "data-source" ;

	protected static final String ATTR_FILE_REF						= "url" ;
	protected static final String ATTR_FILE_TYPE					= "type" ;

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private String dataRoot = null ;
  protected String getDataRoot() { return dataRoot ; }
  protected void setDataRoot( String newValue ) { dataRoot = newValue ; }
  
  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
	/**
	 * Initialize the new instance with a root directory where the data files 
	 * can be found.
	 * 
	 * @param rootPath The root directory from where to look for relative file names
	 * @param configData The configuration read from an XML file
	 */
	public XmlDirectoryConfiguration(String rootPath, Element configData)
	{
		super();
		this.setDataRoot(rootPath);
		this.init(configData);
	} // XmlDirectoryConfiguration() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected void init(Element configData)
	{
		Element[] children;

		if (TAG_ROOT.equals(configData.getName()))
		{
			children = configData.getChildren();
			for (int i = 0; i < children.length; i++)
			{
				if (TAG_CONFIGURATION.equals(children[i].getName()))
				{
					this.processConfiguration(children[i]);
				}
				else if (TAG_META_DATA.equals(children[i].getName()))
				{
					this.processMetaData(children[i]);
				}
				else if (TAG_DATA.equals(children[i].getName()))
				{
					this.processData(children[i]);
				}
			}
		}
	} // init() 

	// -------------------------------------------------------------------------

	protected void processConfiguration(Element configData)
	{
		Element[] children;

		children = configData.getChildren();
		for (int i = 0; i < children.length; i++)
		{
			if (TAG_CONTAINER_RULE.equals(children[i].getName()))
			{
				this.processContainerRule(children[i]);
			}
		}
	} // processConfiguration() 

	// -------------------------------------------------------------------------

	protected void processMetaData(Element metaData)
	{
		Element[] children;

		children = metaData.getChildren();
		for (int i = 0; i < children.length; i++)
		{
			if (TAG_SUFFIX.equals(children[i].getName()))
			{
				this.processSuffix(children[i]);
			}
		}
	} // processMetaData() 

	// -------------------------------------------------------------------------

	protected void processData(Element dataElement)
	{
		Element[] children;

		children = dataElement.getChildren();
		for (int i = 0; i < children.length; i++)
		{
			if (TAG_DATA_SOURCE.equals(children[i].getName()))
			{
				this.processDataSource(children[i]);
			}
		}
	} // processData() 

	// -------------------------------------------------------------------------

	protected void processSuffix(Element suffixElement)
	{
		if (suffixElement.hasText())
		{
			this.addSuffix(suffixElement.getText());
		}
	} // processSuffix() 

	// -------------------------------------------------------------------------

	protected void processDataSource(Element dataSourceElement)
	{
		String url;
		String type;
		DirectoryDataFile dataFile;

		if (dataSourceElement.hasAttributes())
		{
			url = dataSourceElement.getAttribute(ATTR_FILE_REF);
			if (!this.str().isNullOrBlank(url))
			{
				// TODO allow absolute URLs as well
				dataFile = new DirectoryDataFile(FileLocator.create(this.getDataRoot() + "/" + url));
				type = dataSourceElement.getAttribute(ATTR_FILE_TYPE);
				if (!this.str().isNullOrBlank(type))
				{
					dataFile.setFileType(type);
				}
				this.addDataSource(dataFile);
			}
		}
	} // processDataSource() 

	// -------------------------------------------------------------------------

	protected void processContainerRule(Element ruleElement)
	{
		if (ruleElement.hasText())
		{
			this.setContainerRecognizingRule(ruleElement.getText());
		}
	} // processContainerRule() 

	// -------------------------------------------------------------------------

} // class XmlDirectoryConfiguration 
