// ===========================================================================
// CONTENT  : CLASS InterceptorLoader
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 30/03/2012
// HISTORY  :
//  13/01/2012  mdu  CREATED
//	30/03/2012	mdu		changed	-->	Load from XML element (not only from file)
//
// Copyright (c) 2012, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.interceptors ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.pfsw.tools.beluga.common.Constants.*;

import java.util.List;

import org.pf.pax.XPathAccess;
import org.pf.reflect.ReflectUtil;
import org.pf.reflect.XmlReflection;
import org.pf.util.CollectionUtil;
import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.server.IResponseInterceptor;
import org.pfsw.tools.beluga.ldap.server.InterceptorType;
import org.w3c.dom.Element;

/**
 * Loads the interceptors to be used within the server.
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public class InterceptorLoader
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	private static final CollectionUtil CU = CollectionUtil.current();
	private static final ReflectUtil RU = ReflectUtil.current();

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public InterceptorLoader()
  {
    super() ;
  } // InterceptorLoader() 

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public ResponseInterceptorChain loadResponseInterceptors(Element configElement) 
  {
  	IResponseInterceptor[] interceptors;
  	List<Element> elements;
  	Element interceptorsTag;
  	XPathAccess xPathAccess;

  	if (configElement == null)
		{			
  		return new ResponseInterceptorChain();
		}
  	
  	xPathAccess = new XPathAccess(configElement);
		elements = xPathAccess.getElements("//interceptors");
		if (CU.isNullOrEmpty(elements))
		{
			this.logger().logDebug("No //interceptors found under <{0}>", configElement.getNodeName());
			return new ResponseInterceptorChain();
		}
		interceptorsTag = elements.get(0);
  	interceptors = this.loadInterceptors(interceptorsTag, IResponseInterceptor.class);
  	this.logLoadedInterceprors(interceptors);
  	return this.createInterceptorChain(interceptors);
  } // loadResponseInterceptors() 
  
  // -------------------------------------------------------------------------
  
  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected ResponseInterceptorChain createInterceptorChain(IResponseInterceptor[] interceptors) 
  {
  	ResponseInterceptorChain interceptorChain;
  	
  	interceptorChain = new ResponseInterceptorChain();
  	interceptorChain.addInterceptors(interceptors);		
  	return interceptorChain;
  } // createInterceptorChain()
  
  // -------------------------------------------------------------------------
  
  protected <T> T[] loadInterceptors(Element xmlData, Class<T> interceptorType) 
  {
  	XPathAccess xPathAccess;
  	List<Element> elements;
  	List<T> interceptors;
  	XmlReflection xmlReflection;
  	String type;
  	
  	if (xmlData != null)
  	{
  		xPathAccess = new XPathAccess(xmlData);
  		type = RU.getAnnotationValueFrom(interceptorType, InterceptorType.class);
  		elements = xPathAccess.getElements("interceptor-chain[@type='" + type + "']/interceptor");
  		xmlReflection = new XmlReflection();
  		xmlReflection.setAttrNameForFieldValue(null); // get the value from element's text rather than an attribute
  		try
  		{
  			interceptors = xmlReflection.createInitializedInstances(elements, interceptorType);
  			return CU.toArray(interceptors, interceptorType);
  		}
  		catch (NoSuchFieldException ex)
  		{
  			ex.printStackTrace();
  		}
  	}
  	return null;
  } // loadInterceptors() 
  
  // -------------------------------------------------------------------------
  
	protected void logLoadedInterceprors(IResponseInterceptor[] interceptors)
	{
		if (interceptors.length > 0)
		{
			this.logger().logInfo(I_LOADED_INTERCEPTORS, "Loaded interceptors:");
			for (IResponseInterceptor interceptor : interceptors)
			{
				this.logger().logInfo(I_LOADED_INTERCEPTOR, interceptor.toString());
			}
		}
	} // logLoadedInterceprors()
	
	// -------------------------------------------------------------------------
	
	protected LoggerService logger()
	{
		return LoggerService.getLogger();
	} // logger() 

	// -------------------------------------------------------------------------

} // class InterceptorLoader 
