// ===========================================================================
// CONTENT  : ABSTRACT CLASS ResponseMessage
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 08/07/2004
// HISTORY  :
//  08/07/2004  mdu  CREATED
//
// Copyright (c) 2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.messages ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import org.pf.directory.DistinguishedName;
import org.pf.text.StringUtil;

/**
 * Superclass of all response messages
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
abstract public class ResponseMessage extends AbstractMessage
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  protected static final int LDAP_SUCCESS = 0;
  protected static final int LDAP_OPERATIONS_ERROR = 1;
  protected static final int LDAP_PROTOCOL_ERROR = 2;
  protected static final int LDAP_TIME_LIMIT_EXCEEDED = 3;
  protected static final int LDAP_SIZE_LIMIT_EXCEEDED = 4;
  protected static final int LDAP_COMPARE_FALSE = 5;
  protected static final int LDAP_COMPARE_TRUE = 6;
  protected static final int LDAP_AUTH_METHOD_NOT_SUPPORTED = 7;
  protected static final int LDAP_STRONG_AUTH_REQUIRED = 8;
  protected static final int LDAP_PARTIAL_RESULTS = 9;
  protected static final int LDAP_REFERRAL = 10;
  protected static final int LDAP_ADMIN_LIMIT_EXCEEDED = 11;
  protected static final int LDAP_UNAVAILABLE_CRITICAL_EXTENSION = 12;
  protected static final int LDAP_CONFIDENTIALITY_REQUIRED = 13;
  protected static final int LDAP_SASL_BIND_IN_PROGRESS = 14;
  protected static final int LDAP_NO_SUCH_ATTRIBUTE = 16;
  protected static final int LDAP_UNDEFINED_ATTRIBUTE_TYPE = 17;
  protected static final int LDAP_INAPPROPRIATE_MATCHING = 18;
  protected static final int LDAP_CONSTRAINT_VIOLATION = 19;
  protected static final int LDAP_ATTRIBUTE_OR_VALUE_EXISTS = 20;
  protected static final int LDAP_INVALID_ATTRIBUTE_SYNTAX = 21;
  protected static final int LDAP_NO_SUCH_OBJECT = 32;
  protected static final int LDAP_ALIAS_PROBLEM = 33;
  protected static final int LDAP_INVALID_DN_SYNTAX = 34;
  protected static final int LDAP_IS_LEAF = 35;
  protected static final int LDAP_ALIAS_DEREFERENCING_PROBLEM = 36;
  protected static final int LDAP_INAPPROPRIATE_AUTHENTICATION = 48;
  protected static final int LDAP_INVALID_CREDENTIALS = 49;
  protected static final int LDAP_INSUFFICIENT_ACCESS_RIGHTS = 50;
  protected static final int LDAP_BUSY = 51;
  protected static final int LDAP_UNAVAILABLE = 52;
  protected static final int LDAP_UNWILLING_TO_PERFORM = 53;
  protected static final int LDAP_LOOP_DETECT = 54;
  protected static final int LDAP_NAMING_VIOLATION = 64;
  protected static final int LDAP_OBJECT_CLASS_VIOLATION = 65;
  protected static final int LDAP_NOT_ALLOWED_ON_NON_LEAF = 66;
  protected static final int LDAP_NOT_ALLOWED_ON_RDN = 67;
  protected static final int LDAP_ENTRY_ALREADY_EXISTS = 68;
  protected static final int LDAP_OBJECT_CLASS_MODS_PROHIBITED = 69;
  protected static final int LDAP_AFFECTS_MULTIPLE_DSAS = 71;
  protected static final int LDAP_OTHER = 80;

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private int responseCode = 0 ;
  public int getResponseCode() { return responseCode ; }
  public void setResponseCode( int newValue ) { responseCode = newValue ; }

  private String DN = StringUtil.EMPTY_STRING ;
  public String getDN() { return DN ; }
  public void setDN( String newValue ) { DN = newValue ; }

  private String messageText = StringUtil.EMPTY_STRING ;
  public String getMessageText() { return messageText ; }
  public void setMessageText( String newValue ) { messageText = newValue ; }

	// =========================================================================
  // CLASS METHODS
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with a request message.
   * That is useful to copy the message id from the request to the corresponding
   * response.
   */
  protected ResponseMessage( RequestMessage request )
  {
    this( request.getMessageId() ) ;
  } // ResponseMessage() 

  // -------------------------------------------------------------------------

  /**
   * Initialize the new instance with a message id.
   */
  protected ResponseMessage( int msgId )
  {
    super() ;
    this.setMessageId( msgId ) ;
  } // ResponseMessage() 

  // -------------------------------------------------------------------------

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public boolean isBindResponse()
	{
		return false ;
	} // isBindResponse() 

	// -------------------------------------------------------------------------

  public boolean isSearchResponse()
  {
  	return false ;
  } // isSearchResponse() 
  
  // -------------------------------------------------------------------------
  
  public boolean isSearchResultEntry()
	{
		return false ;
	} // isSearchResultEntry() 

	// -------------------------------------------------------------------------

  public boolean isSearchResultDone()
	{
		return false ;
	} // isSearchResultDone() 

	// -------------------------------------------------------------------------

  public boolean isDeleteResponse()
	{
		return false ;
	} // isDeleteResponse() 

	// -------------------------------------------------------------------------

  public boolean isAddResponse()
  {
  	return false ;
  } // isAddResponse() 

  // -------------------------------------------------------------------------

  public boolean isModifyResponse()
  {
  	return false ;
  } // isModifyResponse() 

  // -------------------------------------------------------------------------

  public boolean isBasicResult()
	{
		return false ;
	} // isBasicResult() 

	// -------------------------------------------------------------------------

  /**
   * Returns true if this message is a success message
   */
  public boolean isSuccess()
	{
		return ( this.getResponseCode() == 0 ) ;
	} // isSuccess() 

	// -------------------------------------------------------------------------

  public void invalidDN( String dn )
	{
		this.setResponseCode( LDAP_INVALID_DN_SYNTAX ) ;
		this.setMessageText( "Invalid dn: " + dn ) ;
	} // invalidDN() 

	// -------------------------------------------------------------------------

  public void operationsError( String text )
	{
		this.setResponseCode( LDAP_OPERATIONS_ERROR ) ;
		this.setMessageText( "Unable to process request: " + text ) ;
	} // operationsError() 

	// -------------------------------------------------------------------------

  public void noSuchObject( String dn )
	{
		this.setResponseCode( LDAP_NO_SUCH_OBJECT ) ;
		this.setMessageText( "Unable find object: " + dn ) ;
	} // noSuchObject() 

	// -------------------------------------------------------------------------

  public void noSuchObject( DistinguishedName dn )
  {
  	this.noSuchObject( dn.toString() ) ;
  } // noSuchObject() 

  // -------------------------------------------------------------------------

  public void unwillingToPerform( String text )
	{
		this.setResponseCode( LDAP_UNWILLING_TO_PERFORM ) ;
		this.setMessageText( text ) ;
	} // unwillingToPerform() 

	// -------------------------------------------------------------------------

  public void protocolError( String text )
  {
  	this.setResponseCode( LDAP_PROTOCOL_ERROR ) ;
  	this.setMessageText( text ) ;
  } // protocolError() 

  // -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected void appendParameter( StringBuffer buffer )
	{
		buffer.append( " rc=" ) ;
		buffer.append( this.getResponseCode() ) ;
		if ( ! this.str().isNullOrBlank( this.getMessageText() ) )
		{
			buffer.append( " " ) ;
			buffer.append( this.getMessageText() ) ;
		}
	} // appendParameter() 

	// -------------------------------------------------------------------------

} // class ResponseMessage 
