// ===========================================================================
// CONTENT  : ABSTRACT CLASS LDAPProtocol
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 08/07/2004
// HISTORY  :
//  08/07/2004  mdu  CREATED
//
// Copyright (c) 2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.protocol;

// ===========================================================================
// IMPORTS
// ===========================================================================
import org.pf.text.StringUtil;
import org.pf.util.Bool;
import org.pfsw.tools.beluga.common.LoggerService;


/**
 * Common superclass of LDAP protocol parsers and builders
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
abstract public class LDAPProtocol
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	// For OIDs look at http://www.networksorcery.com/enp/protocol/ldap.htm
	
	protected static final int LDAP_SUPERIOR_DN = 128;
	
  protected static final int LDAP_REQ_BIND = 96;
  protected static final int LDAP_REQ_UNBIND = 66;
  protected static final int LDAP_REQ_SEARCH = 99;
  protected static final int LDAP_REQ_MODIFY = 102;
  protected static final int LDAP_REQ_ADD = 104;
  protected static final int LDAP_REQ_DELETE = 74;
  protected static final int LDAP_REQ_MODRDN = 108;
  protected static final int LDAP_REQ_COMPARE = 110;
  protected static final int LDAP_REQ_ABANDON = 80;
  protected static final int LDAP_REQ_EXTENSION = 119;
  
  protected static final int LDAP_REP_BIND = 97;
  protected static final int LDAP_REP_SEARCH = 100;
  protected static final int LDAP_REP_SEARCH_REF = 115;
  protected static final int LDAP_REP_RESULT = 101;
  protected static final int LDAP_REP_MODIFY = 103;
  protected static final int LDAP_REP_ADD = 105;
  protected static final int LDAP_REP_DELETE = 107;
  protected static final int LDAP_REP_MODRDN = 109;
  protected static final int LDAP_REP_COMPARE = 111;
  protected static final int LDAP_REP_EXTENSION = 120;
  protected static final int LDAP_REP_REFERRAL = 163;
  protected static final int LDAP_REP_EXT_OID = 138;
  protected static final int LDAP_REP_EXT_VAL = 139;

  protected static final int LDAP_CONTROLS_TAG = 160; // = 0xA0 = -96
  
  protected static final int LBER_BOOLEAN = 1;
  protected static final int LBER_INTEGER = 2;
  protected static final int LBER_BITSTRING = 3;
  protected static final int LBER_OCTETSTRING = 4;
  protected static final int LBER_NULL = 5;
  protected static final int LBER_ENUMERATED = 10;
  protected static final int LBER_SEQUENCE = 48;
  protected static final int LBER_SET = 49;
  
  // Filter CHOICE
  protected static final int LDAP_AND									= 160 ; 
  protected static final int LDAP_OR									= 161 ; 
  protected static final int LDAP_NOT									= 162 ; 
  protected static final int LDAP_EQUALITY_MATCH			= 163 ; 
  protected static final int LDAP_SUBSTRINGS					= 164 ; 
  protected static final int LDAP_GREATER_OR_EQUAL		= 165 ; 
  protected static final int LDAP_LESS_OR_EQUAL				= 166 ; 
  protected static final int LDAP_PRESENT							= 135 ; 
  protected static final int LDAP_APPROX_MATCH				= 168 ; 
  protected static final int LDAP_EXTENSIBLE_MATCH		= 169 ; 
  
  
  // LDAPv2 uses ISO-8859-1 ; LDAPv3 uses UTF-8
  protected static final boolean USE_UTF8	= true ; // true = UTF-8, false = ISO-8859-1
  protected static final String CHAR_ENCODING = "UTF-8" ;
  
  protected static final boolean DEBUG_RAW = Bool.isTrue( System.getProperty( "debug.raw" ) ) ;
  
  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================

  // =========================================================================
  // CLASS METHODS
  // =========================================================================


  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  protected LDAPProtocol()
  {
    super() ;
  } // LDAPProtocol()

  // -------------------------------------------------------------------------
  
  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
	protected void debugPrintRawData( byte[] data ) 
	{
		if ( data == null )
		{
			return ;
		}
		for (int i = 0; i < data.length; i++ )
		{
			if ( i > 0 )
			{
				System.out.print( ";" ) ;
			}
			System.out.print( ""+data[i] ) ;
		}
		System.out.println() ;
	} // debugPrintRawData()

	// -------------------------------------------------------------------------
	
	protected boolean isDebuggingRawdataOn() 
	{
		return DEBUG_RAW && this.logger().isDebugOn() ;
	} // isDebugginRawdataOn()

	// -------------------------------------------------------------------------
	
	protected LoggerService logger() 
	{
		return LoggerService.getLogger() ;
	} // logger() 

	// -------------------------------------------------------------------------

	protected StringUtil str()
	{
		return StringUtil.current();
	} // str()

	//-------------------------------------------------------------------------
	
} // class LDAPProtocol
