// ===========================================================================
// CONTENT  : CLASS LDAPResponseBuilder
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 08/07/2004
// HISTORY  :
//  08/07/2004  mdu  CREATED
//
// Copyright (c) 2004, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.protocol;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.Iterator;
import java.util.List;

import org.pf.directory.MultiValueAttributes;
import org.pfsw.tools.beluga.ldap.messages.AddResponse;
import org.pfsw.tools.beluga.ldap.messages.BindResponse;
import org.pfsw.tools.beluga.ldap.messages.DeleteResponse;
import org.pfsw.tools.beluga.ldap.messages.ModifyResponse;
import org.pfsw.tools.beluga.ldap.messages.ResponseMessage;
import org.pfsw.tools.beluga.ldap.messages.ResultResponse;
import org.pfsw.tools.beluga.ldap.messages.SearchResultDone;
import org.pfsw.tools.beluga.ldap.messages.SearchResultEntry;

import com.sun.jndi.ldap.BerEncoder;

/**
 * This builder creates byte arrays that can be transmitted over a connection
 * according to LDAPv3 (RFC-2251).
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class LDAPResponseBuilder extends LDAPProtocol
{
	// =========================================================================
	// CONSTANTS
	// =========================================================================
	protected static final String ERR_RESP_BUILDING = "RES001";

	// =========================================================================
	// INSTANCE VARIABLES
	// =========================================================================

	// =========================================================================
	// CLASS METHODS
	// =========================================================================

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	/**
	 * Initialize the new instance with default values.
	 */
	public LDAPResponseBuilder()
	{
		super();
	} // LDAPResponseBuilder() 

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Converts the given response message to a ASN.1 encoded message according
	 * to the LDAPv3 specification.
	 */
	public byte[] buildResponseData(ResponseMessage response)
	{
		byte[] rawData = null;

		rawData = response.getRawData();
		if (rawData == null)
		{
			if (response.isBindResponse())
			{
				rawData = this.buildBindResponse((BindResponse)response);
			}
			else if (response.isSearchResultEntry())
			{
				rawData = this.buildSearchResultEntry((SearchResultEntry)response);
			}
			else if (response.isSearchResultDone())
			{
				rawData = this.buildSearchResultDone((SearchResultDone)response);
			}
			else if (response.isDeleteResponse())
			{
				rawData = this.buildDeleteResponse((DeleteResponse)response);
			}
			else if (response.isAddResponse())
			{
				rawData = this.buildAddResponse((AddResponse)response);
			}
			else if (response.isModifyResponse())
			{
				rawData = this.buildModifyResponse((ModifyResponse)response);
			}
			else if (response.isBasicResult())
			{
				rawData = this.buildBasicResult((ResultResponse)response);
			}
		}
		if (this.isDebuggingRawdataOn())
		{
			this.debugPrintRawData(rawData);
		}
		return rawData;
	} // buildResponseData() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected byte[] buildBindResponse(BindResponse response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_BIND);
			this.addLDAPResult(encoder, response);
			// optional serverSaslCreds
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building bind response;", e);
			return null;
		}
	} // buildBindResponse() 

	// -------------------------------------------------------------------------

	protected byte[] buildModifyResponse(ModifyResponse response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_MODIFY);
			this.addLDAPResult(encoder, response);
			// optional serverSaslCreds
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building modify response;", e);
			return null;
		}
	} // buildModifyResponse() 

	// -------------------------------------------------------------------------

	protected byte[] buildDeleteResponse(DeleteResponse response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_DELETE);
			this.addLDAPResult(encoder, response);
			// optional serverSaslCreds
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building delete response;", e);
			return null;
		}
	} // buildDeleteResponse() 

	// -------------------------------------------------------------------------

	protected byte[] buildAddResponse(AddResponse response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_ADD);
			this.addLDAPResult(encoder, response);
			// optional serverSaslCreds
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building add response;", e);
			return null;
		}
	} // buildAddResponse() 

	// -------------------------------------------------------------------------

	protected byte[] buildSearchResultEntry(SearchResultEntry response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_SEARCH);
			encoder.encodeOctetString(response.getDN().getBytes(), LBER_OCTETSTRING);
			this.addPartialAttributeList(encoder, response.getAttributes());
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building search result entry response;", e);
			return null;
		}
	} // buildSearchResultEntry() 

	// -------------------------------------------------------------------------

	protected byte[] buildSearchResultDone(SearchResultDone response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_RESULT);
			this.addLDAPResult(encoder, response);
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building search result done response;", e);
			return null;
		}
	} // buildSearchResultDone() 

	// -------------------------------------------------------------------------

	protected byte[] buildBasicResult(ResultResponse response)
	{
		BerEncoder encoder;

		try
		{
			encoder = this.createEncoderOn(response);
			encoder.beginSeq(LDAP_REP_RESULT);
			this.addLDAPResult(encoder, response);
			encoder.endSeq();
			return this.finalizeEncoding(encoder);
		}
		catch (Exception e)
		{
			this.logger().logError(ERR_RESP_BUILDING, "Building LDAP result ;", e);
			return null;
		}
	} // buildBasicResult() 

	// -------------------------------------------------------------------------

	/**
	 * Creates a response encoder and initializes it with the message id from the
	 * given response message.
	 */
	protected BerEncoder createEncoderOn(ResponseMessage response) throws Exception
	{
		BerEncoder encoder;

		encoder = new BerEncoder();
		encoder.beginSeq(LBER_SEQUENCE);
		encoder.encodeInt(response.getMessageId());
		return encoder;
	} // createEncoderOn() 

	// -------------------------------------------------------------------------

	protected void addPartialAttributeList(BerEncoder encoder, MultiValueAttributes attributes) throws Exception
	{
		String[] attrNames;
		List values;
		Iterator iter;
		Object value;

		encoder.beginSeq(LBER_SEQUENCE);
		attrNames = attributes.getAttributeNames();
		for (int i = 0; i < attrNames.length; i++)
		{
			encoder.beginSeq(LBER_SEQUENCE);
			encoder.encodeOctetString(attrNames[i].getBytes(), LBER_OCTETSTRING);
			encoder.beginSeq(LBER_SET);
			values = attributes.getValues(attrNames[i]);
			iter = values.iterator();
			while (iter.hasNext())
			{
				value = iter.next();
				encoder.encodeOctetString(value.toString().getBytes(), LBER_OCTETSTRING);
			}
			encoder.endSeq(); // SET
			encoder.endSeq(); // SEQUENCE
		}
		encoder.endSeq();
	} // addPartialAttributeList() 

	// -------------------------------------------------------------------------

	protected void addLDAPResult(BerEncoder encoder, ResponseMessage response) throws Exception
	{
		encoder.encodeInt(response.getResponseCode(), LBER_ENUMERATED);
		encoder.encodeOctetString(response.getDN().getBytes(), LBER_OCTETSTRING);
		encoder.encodeOctetString(response.getMessageText().getBytes(), LBER_OCTETSTRING);
	} // addLDAPResult() 

	// -------------------------------------------------------------------------

	protected byte[] finalizeEncoding(BerEncoder encoder) throws Exception
	{
		byte[] responseData;

		encoder.endSeq();
		responseData = encoder.getTrimmedBuf();
		return responseData;
	} // finalizeEncoding() 

	// -------------------------------------------------------------------------

} // class LDAPResponseBuilder 
