// ===========================================================================
// CONTENT  : CLASS CommunicationChannel
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 16/05/2010
// HISTORY  :
//  11/07/2004  mdu  CREATED
//	16/05/2010	mdu		changed	-->	handling of parse requests and wrong protocol version
//
// Copyright (c) 2004-2010, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.server;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.util.concurrent.atomic.AtomicBoolean;

import org.pf.bif.execution.IService;
import org.pf.bif.execution.IServiceStateChangeListener;
import org.pf.net.BinarySocketConnection;
import org.pf.net.RequestProcessor;
import org.pf.util.StopWatch;
import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.messages.BindResponse;
import org.pfsw.tools.beluga.ldap.messages.RequestMessage;
import org.pfsw.tools.beluga.ldap.messages.ResponseMessage;
import org.pfsw.tools.beluga.ldap.messages.ResultResponse;
import org.pfsw.tools.beluga.ldap.messages.SearchResponse;
import org.pfsw.tools.beluga.ldap.protocol.LDAPRequestException;
import org.pfsw.tools.beluga.ldap.protocol.LDAPRequestParser;
import org.pfsw.tools.beluga.ldap.protocol.LDAPResponseBuilder;
import org.pfsw.tools.beluga.ldap.protocol.LDAPUnknownRequestException;
import org.pfsw.tools.beluga.ldap.protocol.LDAPVersionNotSupportedException;

/**
 * Each communication channel is opened in a separate thread. It holds a
 * connection to the client. When the connection gets closed the thread is
 * finished as well.   <br>
 * The communication channel is responsible to receive the raw request data
 * and transfer it to request message objects. Then it creates a Transaction
 * on itself and the request message and sends it to the LDAPEngine for
 * processing. <br>
 * Finally it converts response messages to raw protocol data and sends them
 * back to the client over the connection.
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public class CommunicationChannel implements RequestProcessor, IService
{
	// =========================================================================
	// CONSTANTS
	// =========================================================================
	protected static final String E_RESPONSE_NOT_CONVERTED = "CCH0001";
	protected static final String W_NO_RESPONSE = "CCH0002";
	protected static final String I_CONNECTION_OPENED = "CCH0003";
	protected static final String I_CONNECTION_CLOSED = "CCH0004";
	protected static final String E_UNKNOWN_REQUEST = "CCH0005";
	protected static final String E_UNSUPPORTED_PROTOCOL_VERSION = "CCH0006";
	protected static final String E_PROCESSING_PROBLEM = "CCH0007";
	protected static final String I_PROCESS_TIME = "CCH0008";

	// =========================================================================
	// INSTANCE VARIABLES
	// =========================================================================
	private LDAPEngine ldapEngine = null;
	protected LDAPEngine getLdapEngine() { return ldapEngine; }
	protected void setLdapEngine(LDAPEngine newValue) { ldapEngine = newValue; }
	
  private LDAPServerConfiguration config = null ;
  protected LDAPServerConfiguration getConfig() { return config ; }
  protected void setConfig( LDAPServerConfiguration newValue ) { config = newValue ; }

	private BinarySocketConnection connection = null;
	protected BinarySocketConnection connection() { return connection; }
	protected void connection(BinarySocketConnection newValue) { connection = newValue; }

	private AtomicBoolean continueListening = new AtomicBoolean(true);
	protected boolean continueListening() { return continueListening.get(); }
	protected void continueListening(boolean newValue) { continueListening.set(newValue); }

	private LDAPResponseBuilder responseBuilder = new LDAPResponseBuilder();
	protected LDAPResponseBuilder getResponseBuilder() { return responseBuilder; }
	protected void setResponseBuilder(LDAPResponseBuilder newValue)	{	responseBuilder = newValue;	}

	private IServiceStateChangeListener stateChangeListener = null;
	protected IServiceStateChangeListener getStateChangeListener() {	return stateChangeListener;	}
	public void setStateChangeListener(IServiceStateChangeListener newValue) { stateChangeListener = newValue; }

	// =========================================================================
	// CONSTRUCTORS
	// =========================================================================
	public CommunicationChannel(LDAPServerConfiguration config, LDAPEngine ldapEngine, BinarySocketConnection conn)
	{
		super();
		this.setConfig(config);
		this.setLdapEngine(ldapEngine);
		this.connection(conn);
	} // CommunicationChannel() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Runs the processing of requests coming in from the underlying
	 * socket connection.
	 */
	public void run()
	{
		byte[] requestData;

		this.logger().logInfo(I_CONNECTION_OPENED, ">>> {0} started...", Thread.currentThread().getName());
		this.stateChanged(IServiceStateChangeListener.SERVICE_STARTED);

		try
		{
			while (this.continueListening())
			{
				if (this.connection().isOpen())
				{
					requestData = this.connection().getData();
					if (requestData != null)
					{
						this.processRequest(requestData);
					}
				}
				else
				{
					try
					{
						Thread.sleep(10);
					}
					catch (InterruptedException e)
					{
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					this.stopListening();
				}
			}
		}
		finally
		{
			this.connection().close();

			this.logger().logInfo(I_CONNECTION_CLOSED, "<<< {0} closed", Thread.currentThread().getName());
			this.stateChanged(IServiceStateChangeListener.SERVICE_STOPPED);
		}
	} // run() 

	// -------------------------------------------------------------------------

	/**
	 * Stops listening to the open socket and end the loop in run which actually
	 * ends the thread which executed this channel.
	 */
	public void terminate()
	{
		this.stopListening();
		this.connection().close(); // To interrupt blocking read on socket
	} // terminate() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected void processRequest(byte[] data)
	{
		LDAPRequestParser parser;
		RequestMessage request;
		Transaction transaction;
		StopWatch stopWatch = new StopWatch();

		parser = new LDAPRequestParser();
		try
		{
			stopWatch.start();
			request = parser.parseRequest(data);
			transaction = new Transaction(this, request);
			this.getLdapEngine().processTransaction(transaction);
			this.handleResponse(transaction);
			stopWatch.stop();
			this.logger().logInfo(I_PROCESS_TIME, "Executed {0} in {1} ms. ({2})", 
					request.getClass().getSimpleName(), stopWatch.getDuration(), Thread.currentThread().getName());
		}
		catch (LDAPUnknownRequestException e)
		{
			this.logger().logError(E_UNKNOWN_REQUEST, "Unknown request {0}.", e.getRequestId());
			this.sendOperationNotSupportedResponse(e);
		}
		catch (LDAPVersionNotSupportedException e)
		{
			this.logger().logError(E_UNSUPPORTED_PROTOCOL_VERSION, "LDAP protocol version {0} not supported.", e.getVersion());
			this.sendProtocolErrorResponse(e);
			// A failed bind results in closing the socket and drop the connection
			this.stopListening();
		}
		catch (LDAPRequestException e)
		{
			this.logger().logError(E_PROCESSING_PROBLEM, "Processing problem {0}.", e.getMessage());
			this.sendProtocolErrorResponse(e);
		}
	} // processRequest() 

	// -------------------------------------------------------------------------

	/**
	 * @param transaction
	 */
	protected void handleResponse(Transaction transaction)
	{
		BindResponse bindResponse;

		if (transaction.hasSingleResponse())
		{
			this.sendResponse(transaction.getResponse());
			if (transaction.getResponse().isBindResponse())
			{
				bindResponse = (BindResponse)transaction.getResponse();
				if (!bindResponse.isSuccess())
				{
					// A failed bind results in closing the socket and drop the connection
					this.stopListening();
				}
			}
		}
		else
		{
			if (transaction.getRequest().isUnbindRequest())
			{
				this.stopListening();
			}
			else
			{
				if (transaction.getRequest().isSearchRequest())
				{
					this.sendSearchResult(transaction.getSearchResult());
				}
				else
				{
					// TODO Write log for invalid response and return something to client

				}
			}
		}
	} // handleResponse() 

	// -------------------------------------------------------------------------

	/**
	 * Finish the transaction, that is the response will be sent to the client.
	 */
	protected void sendResponse(final ResponseMessage response)
	{
		byte[] rawData = null;
		ResponseMessage responseMessage;

		if (response == null)
		{
			this.logger().logWarning(W_NO_RESPONSE, "No response received");
		}
		else
		{
			responseMessage = this.getResponseInterceptorChain().handleResponse(response);
			this.logger().logDebug(responseMessage.toString());
			rawData = this.getResponseBuilder().buildResponseData(responseMessage);
			if (rawData == null)
			{
				this.logger().logError(E_RESPONSE_NOT_CONVERTED, "Cannot convert {0}", responseMessage);
			}
		}
		if (rawData == null)
		{
			// TODO Write warning and create a response raw data for operationsError
		}
		this.connection().writeData(rawData);
	} // sendResponse() 

	// -------------------------------------------------------------------------

	/**
	 * Send all search results back to the client
	 */
	protected void sendSearchResult(SearchResponse[] responses)
	{
		if (responses.length == 0)
		{
			this.logger().logWarning(W_NO_RESPONSE, "No search response");
		}
		else
		{
			for (int i = 0; i < responses.length; i++)
			{
				this.sendResponse(responses[i]);
			}
		}
	} // sendSearchResult() 

	// -------------------------------------------------------------------------

	protected void sendOperationNotSupportedResponse(LDAPUnknownRequestException exception)
	{
		ResultResponse msg;

		msg = new ResultResponse(exception.getMessageId());
		msg.unwillingToPerform("Unsupported request (id=" + exception.getRequestId() + ")");
		this.sendResponse(msg);
	} // sendOperationNotSupportedResponse() 

	// -------------------------------------------------------------------------

	protected void sendProtocolErrorResponse(LDAPRequestException exception)
	{
		ResultResponse msg;

		msg = new ResultResponse(exception.getMessageId());
		msg.protocolError(exception.getMessage());
		this.sendResponse(msg);
	} // sendProtocolErrorResponse() 

	// -------------------------------------------------------------------------

	protected void stopListening()
	{
		this.continueListening(false);
	} // stopListening() 

	// -------------------------------------------------------------------------

	protected void stateChanged(int event)
	{
		if (this.getStateChangeListener() != null)
		{
			this.getStateChangeListener().serviceStateChanged(this, event);
		}
	} // stateChanged() 

	// -------------------------------------------------------------------------
	
	protected IResponseInterceptor getResponseInterceptorChain() 
	{
		return this.getConfig().getResponseInterceptorChain();
	} // getResponseInterceptorChain() 
	
	// -------------------------------------------------------------------------
	
	protected LoggerService logger()
	{
		return LoggerService.getLogger();
	} // logger() 

	// -------------------------------------------------------------------------

} // class CommunicationChannel 
