// ===========================================================================
// CONTENT  : CLASS LDAPEngine
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.2 - 30/06/2006
// HISTORY  :
//  11/07/2004  mdu  CREATED
//	23/06/2006	mdu		changed	-->	to not being a singleton anymore
//	30/06/2006	mdu		added		-->	delete, add
//
// Copyright (c) 2004-2006, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.server ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.io.IOException;

import org.pf.six.ModelConverter;
import org.pfsw.tools.beluga.ldap.directory.Directory;
import org.pfsw.tools.beluga.ldap.directory.XmlDirectoryConfiguration;
import org.pfsw.tools.beluga.ldap.messages.AddRequest;
import org.pfsw.tools.beluga.ldap.messages.BindRequest;
import org.pfsw.tools.beluga.ldap.messages.DeleteRequest;
import org.pfsw.tools.beluga.ldap.messages.ModifyRequest;
import org.pfsw.tools.beluga.ldap.messages.RequestMessage;
import org.pfsw.tools.beluga.ldap.messages.ResponseMessage;
import org.pfsw.tools.beluga.ldap.messages.SearchRequest;
import org.pfsw.tools.beluga.ldap.messages.SearchResponse;
import org.pfsw.tools.beluga.ldap.messages.UnbindRequest;
import org.w3c.dom.Element;

/**
 * An instance of this class is responsible for invoking all configured
 * request processors on each incoming request.
 *
 * @author Manfred Duchrow
 * @version 1.2
 */
class LDAPEngine
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private LDAPv3RequestProcessor[] requestProcessors = null ;
  protected LDAPv3RequestProcessor[] getRequestProcessors() { return requestProcessors ; }
  protected void setRequestProcessors( LDAPv3RequestProcessor[] newValue ) { requestProcessors = newValue ; }
  /* TODO Implement response processors
  private List responseProcessors = null ;
  protected List getResponseProcessors() { return responseProcessors ; }
  protected void setResponseProcessors( List newValue ) { responseProcessors = newValue ; }
  */  
  
  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
	/**
	 * Initialize the new instance with default values.
	 * @throws IOException 
	 */
	protected LDAPEngine(LDAPServerConfiguration config) throws IOException
	{
		super();
		this.init(config);
	} // LDAPEngine()

	// -------------------------------------------------------------------------

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	public void processTransaction(Transaction transaction)
	{
		LDAPv3RequestProcessor[] reqProcessors;
		RequestMessage request;
		ResponseMessage response = null;
		SearchResponse[] responses = null;
		int i;

		request = transaction.getRequest();
		reqProcessors = this.getRequestProcessors();
		i = 0;
		while ((i < reqProcessors.length) && (response == null))
		{
			if (request.isBindRequest()) // >>>>>>>>>>>>>>>>>>>>>>>>>> BIND
			{
				response = reqProcessors[i].bind((BindRequest)request);
				if (response != null)
				{
					transaction.setResponse(response);
				}
			}
			else if (request.isSearchRequest()) // >>>>>>>>>>>>>>>>>>>>>>>> SEARCH
			{
				responses = reqProcessors[i].search((SearchRequest)request);
				if (responses != null)
				{
					for (int j = 0; j < responses.length; j++)
					{
						transaction.setSearchResult(responses);
					}
				}
			}
			else if (request.isDeleteRequest()) // >>>>>>>>>>>>>>>>>>>>>>>> DELETE
			{
				response = reqProcessors[i].delete((DeleteRequest)request);
				if (response != null)
				{
					transaction.setResponse(response);
				}
			}
			else if (request.isAddRequest()) // >>>>>>>>>>>>>>>>>>>>>>>> ADD
			{
				response = reqProcessors[i].add((AddRequest)request);
				if (response != null)
				{
					transaction.setResponse(response);
				}
			}
			else if (request.isModifyRequest()) // >>>>>>>>>>>>>>>>>>>>>>>> MODIFY
			{
				response = reqProcessors[i].modify((ModifyRequest)request);
				if (response != null)
				{
					transaction.setResponse(response);
				}
			}
			else if (request.isUnbindRequest()) // >>>>>>>>>>>>>>>>>>>>>>>> UNBIND
			{
				reqProcessors[i].unbind((UnbindRequest)request);
			}
			i++;
		}
		i = 0;
		// TODO while loop over response processors
	} // processTransaction()

	// -------------------------------------------------------------------------

	/**
	 * Stops this LDAP processing engine. That is all resources will be released.
	 * Afterwards no processing calls to this LDAP engine are allowed anymore.
	 */
	public void stop()
	{
		// TODO Cleanup
	} // stop()

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected void init(LDAPServerConfiguration config) throws IOException
	{
		LDAPv3RequestProcessor[] reqProcessors;
		Element configData;
		XmlDirectoryConfiguration dirConfig;
		ModelConverter modelConverter = new ModelConverter();

		configData = config.getDirectoryContentConfiguration();
		dirConfig = new XmlDirectoryConfiguration(config.getDataRootPath(), modelConverter.toSIXElement(configData));

		reqProcessors = new LDAPv3RequestProcessor[1];
		reqProcessors[0] = new Directory(dirConfig);
		this.setRequestProcessors(reqProcessors);
		// TODO this.setResponseProcessors( ... ) ;

	} // init()

	// -------------------------------------------------------------------------

} // class LDAPEngine
