// ===========================================================================
// CONTENT  : CLASS Beluga
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.2 - 23/06/2006
// HISTORY  :
//  25/06/2004  mdu  CREATED
//	07/08/2004	mdu		changed	-->	Moved from main class
//	23/06/2006	mdu		changed	-->	Gets initialized from LDAPServerConfiguration
//
// Copyright (c) 2004-2006, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.server;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import org.pf.bif.execution.IService;
import org.pf.bif.execution.IServiceStateChangeListener;
import org.pf.bif.logic.ICondition;
import org.pf.net.BinaryRequestServer;
import org.pf.net.BinarySocketConnection;
import org.pf.net.RequestProcessor;
import org.pf.util.Countdown;
import org.pfsw.tools.beluga.common.LoggerService;

/**
 * The LDAP server that is listening to a port for incoming requests.
 * Then it launches a new thread for each request to be processed.
 *
 * @author Manfred Duchrow
 * @version 1.2
 */
public class LDAPServer extends BinaryRequestServer implements IServiceStateChangeListener
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	protected static final String I_SERVER_LISTENING		= "BLS0001" ;
	protected static final String I_SERVER_STOPPED			= "BLS0002" ;
	protected static final String E_PORT_IN_USE					= "BLS0003" ;

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
	private AtomicLong connIdCounter = new AtomicLong(1); 
	private AtomicInteger openConnectionsCounter = new AtomicInteger(0); 
	
  private LDAPServerConfiguration config = null ;
  protected LDAPServerConfiguration getConfig() { return config ; }
  protected void setConfig( LDAPServerConfiguration newValue ) { config = newValue ; }
  
  private LDAPEngine ldapEngine = null ;
  protected LDAPEngine getLdapEngine() { return ldapEngine ; }
  protected void setLdapEngine( LDAPEngine newValue ) { ldapEngine = newValue ; }
  
  private List<CommunicationChannel> commChannels = Collections.synchronizedList(new ArrayList<CommunicationChannel>(100));
  protected List<CommunicationChannel> getCommChannels() { return commChannels ; }
  protected void setCommChannels( List<CommunicationChannel> newValue ) { commChannels = newValue ; }
  
	// =========================================================================
  // CONSTRUCTORS
  // =========================================================================
	/**
	 * Initialize the new instance with default values.
	 * @throws IOException 
	 */
	public LDAPServer() throws IOException
	{
		this(new LDAPServerConfiguration());
	} // LDAPServer() 

	// -------------------------------------------------------------------------

	/**
	 * Initialize the new instance with a port to listen to.
	 * 
	 * @param aPort The port to listen on for new requests
	 * @throws IOException 
	 */
	public LDAPServer(LDAPServerConfiguration configuration) throws IOException
	{
		super(configuration.getPort(), configuration.getIdleTimeout());
		this.setConfig(configuration);
		this.setLdapEngine(new LDAPEngine(config));
	} // LDAPServer() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PUBLIC INSTANCE METHODS
	// =========================================================================
	/**
	 * Starts running the server.
	 */
	public void run()
	{
		if (this.initialize())
		{
			this.logger().logInfo(I_SERVER_LISTENING, "LDAP server listening on port {0}", Integer.toString(this.port()));
			try
			{
				this.listen();
			}
			finally
			{
				this.close();
			}
		}
		else
		{
			this.logger()
					.logInfo(E_PORT_IN_USE, "Cannot open server socket on port {0}. LDAP Server probably already running.",
							Integer.toString(this.port()));
		}
	} // run() 

	// -------------------------------------------------------------------------

	/**
	 * Stops the server
	 */
	public void stop()
	{
		ICondition condition;

		this.close();
		this.closeAllCommunicationChannels();
		condition = new ICondition()
		{
			public boolean isTrue()
			{
				return getCommChannels().isEmpty();
			}
		};
		Countdown.whileFalse(condition, 10000);
		this.getLdapEngine().stop();
		this.logger().logInfo(I_SERVER_STOPPED, "LDAP server stopped.");
	} // stop() 

	// -------------------------------------------------------------------------

	/**
	 * Returns the current number of open connections from clients. 
	 */
	public int getNumberOfOpenConnections() 
	{
		return this.openConnectionsCounter.get();
	} // getNumberOfOpenConnections()
	
	// -------------------------------------------------------------------------
	
	// =========================================================================
	// Interface IServiceStateChangeListener
	// =========================================================================
	public void serviceStateChanged(IService service, int event)
	{
		if (service instanceof CommunicationChannel)
		{
			if (event == IServiceStateChangeListener.SERVICE_STOPPED)
			{
				this.getCommChannels().remove(service);
			}
		}
	} // serviceStateChanged() 

	// -------------------------------------------------------------------------

	// =========================================================================
	// PROTECTED INSTANCE METHODS
	// =========================================================================
	protected RequestProcessor createRequestProcessor(BinarySocketConnection connection)
	{
		CommunicationChannel channel;

		channel = new CommunicationChannel(this.getConfig(), this.getLdapEngine(), connection);
		this.getCommChannels().add(channel);
		channel.setStateChangeListener(this);
		return channel;
	} // createRequestProcessor() 

	// -------------------------------------------------------------------------

	protected Thread createProcessingThread(RequestProcessor requestProcessor)
	{
		Thread thread;

		thread = new Thread(requestProcessor) 
		{
			public void run() 
			{
				super.run();
				connectionClosed();
			}
		};
		thread.setName("Connection-" + this.connIdCounter.getAndIncrement());
		this.connIdCounter.compareAndSet((Long.MAX_VALUE - 10), 1);
		this.openConnectionsCounter.incrementAndGet();
		return thread;
	} // createProcessingThread() 

	// -------------------------------------------------------------------------
	
	protected void connectionClosed() 
	{
		this.openConnectionsCounter.decrementAndGet();		
	} // connectionClosed()
	
	// -------------------------------------------------------------------------

	protected void closeAllCommunicationChannels()
	{
		CommunicationChannel[] channels;

		channels = this.getCommChannels().toArray(new CommunicationChannel[0]);
		for (int i = 0; i < channels.length; i++)
		{
			channels[i].terminate();
		}
	} // closeAllCommunicationChannels() 

	// -------------------------------------------------------------------------

	protected LoggerService logger()
	{
		return LoggerService.getLogger();
	} // logger() 

	// -------------------------------------------------------------------------

} // class LDAPServer 
