// ===========================================================================
// CONTENT  : CLASS LDAPServerConfiguration
// AUTHOR   : M.Duchrow
// VERSION  : 2.0 - 30/03/2012
// HISTORY  :
//  23/06/2006  mdu  CREATED
//	30/03/2012	mdu		changed	-->	extended to load whole config from a single xml file 
//
// Copyright (c) 2006-2012, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.tools.beluga.ldap.server ;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.pfsw.tools.beluga.common.Constants.*;

import java.util.List;

import org.pf.file.FileLocator;
import org.pf.pax.XPathAccess;
import org.pf.pax.XmlUtil;
import org.pf.util.CollectionUtil;
import org.pfsw.tools.beluga.common.LoggerService;
import org.pfsw.tools.beluga.ldap.interceptors.InterceptorLoader;
import org.pfsw.tools.beluga.ldap.interceptors.ResponseInterceptorChain;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Contains all configuration for the LDAP server.
 *
 * @author M.Duchrow
 * @version 2.0
 */
public class LDAPServerConfiguration
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
	private static final XmlUtil XU = XmlUtil.current();
	private static final CollectionUtil CU = CollectionUtil.current();

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private XPathAccess serverConfig = null ;
  private XPathAccess getServerConfig() { return serverConfig ; }
  private void setServerConfig( XPathAccess newValue ) { serverConfig = newValue ; }
  
  private int port = DEFAULT_LDAP_PORT ;
  public int getPort() { return port ; }
  public void setPort( int newValue ) { port = newValue ; }
  
  private FileLocator dirConfigUrl = null;
  protected FileLocator getDirConfigUrl() { return dirConfigUrl ; }
  
  private FileLocator respInterceptorsConfigUrl = null ;
  protected FileLocator getRespInterceptorsConfigUrl() { return respInterceptorsConfigUrl ; }
  public void setRespInterceptorsConfigUrl( FileLocator newValue ) { respInterceptorsConfigUrl = newValue ; }
  
  private ResponseInterceptorChain responseInterceptorChain = null;
  protected void setResponseInterceptorChain( ResponseInterceptorChain newValue ) { responseInterceptorChain = newValue ; }
  
  private int idleTimeout = 0 ;
  public int getIdleTimeout() { return idleTimeout ; }
  public void setIdleTimeout( int newValue ) { idleTimeout = newValue ; }
  
  private String dataRootPath = null ;
  protected String getDataRootPath() { return dataRootPath ; }
  protected void setDataRootPath( String newValue ) { dataRootPath = newValue ; }

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public LDAPServerConfiguration()
  {
    super() ;
  } // LDAPServerConfiguration() 

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
	public void initFrom(FileLocator xmlConfigFile)
	{
		Element xmlConfig;
		List<Element> serverConfigs;
		XPathAccess xPathAccess;
		Element configElement;
  	FileLocator parentDir ;
  	
  	parentDir = xmlConfigFile.getParent();
		this.setDataRootPath(parentDir.getStandardizedAbsolutePath());
		xmlConfig = this.readXmlDataFromFile(xmlConfigFile);
		if (xmlConfig == null)
		{
			return;
		}
		xPathAccess = new XPathAccess(xmlConfig);
		serverConfigs = xPathAccess.getElements("//ldap-server");
		if (CU.isNullOrEmpty(serverConfigs))
		{
			return;
		}
		this.setServerConfig(new XPathAccess(serverConfigs.get(0)));
		
		configElement = this.getXmlConfigElement("server-config/ldap-port");
		if (configElement != null)
		{
			this.setPort(Integer.parseInt(configElement.getTextContent()));
		}
		configElement = this.getXmlConfigElement("server-config/idle-timeout");
		if (configElement != null)
		{
			Integer seconds = Integer.parseInt(configElement.getTextContent());
			this.setIdleTimeout(seconds * 1000); // internally as milliseconds
		}
	} // initFrom() 
	
	// -------------------------------------------------------------------------

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected ResponseInterceptorChain getResponseInterceptorChain()
  {
  	if (responseInterceptorChain == null)
		{
			this.loadResponseInterceptors();
		}
  	return responseInterceptorChain;
  } // getResponseInterceptorChain() 
  
  // -------------------------------------------------------------------------
  
  protected Element getDirectoryContentConfiguration() 
	{
  	Element configElement = null;

  	if (this.getDirConfigUrl() != null)
		{			
  		return this.readXmlDataFromFile(this.getDirConfigUrl());
		}
  	configElement = this.getXmlConfigElement("directory");
  	if (configElement == null)
		{
  		return this.readXmlDataFromFile(FileLocator.create(DEFAULT_DIRECTORY_DEFINITION));			
		}
  	return configElement;
	} // getDirectoryContentConfiguration() 
	
	// -------------------------------------------------------------------------
  
  protected void loadResponseInterceptors() 
	{
		InterceptorLoader loader;
		ResponseInterceptorChain responseChain;
		Element configElement = null;

		if (this.getRespInterceptorsConfigUrl() != null)
		{
			this.logger().logInfo(I_LOADING_INTERCEPTORS, "Loading interceptors from {0}", this.getRespInterceptorsConfigUrl().getAbsolutePath());
			configElement = this.readXmlDataFromFile(this.getRespInterceptorsConfigUrl());
		}
		if (configElement == null)
		{
			configElement = this.getXmlConfigElement("interceptors");
		}
		loader = new InterceptorLoader();
		responseChain = loader.loadResponseInterceptors(configElement);
		this.setResponseInterceptorChain(responseChain);
	} // loadResponseInterceptors() 
	
	// -------------------------------------------------------------------------

  protected Element getXmlConfigElement(String xPathExpression) 
	{
  	if (this.getServerConfig() == null)
		{
			return null;
		}
		return this.getServerConfig().getElement(xPathExpression);
	} // getXmlConfigElement() 
	
	// -------------------------------------------------------------------------
  
  protected Element readXmlDataFromFile(FileLocator file) 
	{
  	Document document;
		try
		{
			document = XU.readXmlFile(file.getAbsolutePath());
			return document.getDocumentElement();
		}
		catch (Exception ex)
		{
			this.logger().logError(E_XML_FILE_PARSING, "Failed to load XML configuration from file: " + file.getAbsolutePath(), ex);
			return null;
		}
	} // readXmlDataFromFile() 
	
  // -------------------------------------------------------------------------

	public void setDirConfigUrl(FileLocator configFile)
	{
		FileLocator parent;
		
		dirConfigUrl = configFile;
		parent = configFile.getParent();
		this.setDataRootPath(parent.getAbsolutePath());
	} // setDirConfigUrl()
  
	// -------------------------------------------------------------------------
	
	protected LoggerService logger()
	{
		return LoggerService.getLogger();
	} // logger() 

	// -------------------------------------------------------------------------

} // class LDAPServerConfiguration 
